/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2002-2005 FPMD-CPV groups
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!

!------------------------------------------------------------------------------!
  MODULE cp_electronic_mass
!------------------------------------------------------------------------------!

    !  This module contains variable and functions relative to the
    !  Car-Parrinello fictitious electronic masse

      USE kinds, ONLY: DP
!
      IMPLICIT NONE
      SAVE

      REAL(DP) :: emass        = 1.0d0    !  fictitious electronic mass ( mu )
      REAL(DP) :: emass_cutoff = 1.0d0    !  kinetic energy cutoff for plane
                                           !  waves to be used for Fourier acceleration
                                           !  preconditioning

!------------------------------------------------------------------------------!
  CONTAINS
!------------------------------------------------------------------------------!

    SUBROUTINE emass_precond( ema0bg, ggp, ngw, tpiba2, emaec )
      USE control_flags, ONLY: iverbosity
      IMPLICIT NONE
      REAL(DP), INTENT(OUT) :: ema0bg(:)
      REAL(DP), INTENT(IN) :: ggp(:), tpiba2, emaec
      INTEGER, INTENT(IN) :: ngw
      INTEGER :: i
      !  mass preconditioning: ema0bg(i) = ratio of emass(g=0) to emass(g)
      !  for g**2>emaec the electron mass ema0bg(g) rises quadratically
      do i = 1, ngw
         ema0bg(i) = 1.0d0 / MAX( 1.d0, tpiba2 * ggp(i) / emaec )
         IF( iverbosity > 2 ) print *,i,' ema0bg(i) ',ema0bg(i)
      end do

      RETURN
    END SUBROUTINE emass_precond


!------------------------------------------------------------------------------!
  END MODULE cp_electronic_mass
!------------------------------------------------------------------------------!
