/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2010 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!=----------------------------------------------------------------------------=!
   MODULE smallbox_grid_dim
!=----------------------------------------------------------------------------=!

     !  Dimensions of the 3D real and reciprocal space FFT subgrids
     !  used for atomic augmentation charge density (USPP)
     !  Dependencies:
     !     fft_scalar       good_fft_dimension, good_fft_order
     !     io_global        stdout, ionode
     !

     IMPLICIT NONE
     SAVE

     PRIVATE
     PUBLIC :: smallbox_grid_init, smallbox_grid_info

   CONTAINS

     SUBROUTINE smallbox_grid_init( dfftp, dfftb )
       !
       USE fft_scalar, only: good_fft_dimension, good_fft_order
       USE fft_types,  only: fft_dlay_descriptor
       !
       IMPLICIT NONE
       !
       TYPE(fft_dlay_descriptor), INTENT(IN)    :: dfftp
       TYPE(fft_dlay_descriptor), INTENT(INOUT) :: dfftb
       !
       ! no default values for grid box: if nr*b=0, ignore

       IF( dfftb%nr1 > 0 .AND. dfftb%nr2 > 0 .AND. dfftb%nr3 > 0 ) THEN

          dfftb%nr1 = good_fft_order( dfftb%nr1 )
          dfftb%nr2 = good_fft_order( dfftb%nr2 )
          dfftb%nr3 = good_fft_order( dfftb%nr3 )
          dfftb%nr1x = good_fft_dimension( dfftb%nr1 )

       ELSE
 
          dfftb%nr1x = dfftb%nr1

       END IF

       dfftb%nr2x = dfftb%nr2
       dfftb%nr3x = dfftb%nr3
       dfftb%nnr  = dfftb%nr1x * dfftb%nr2x * dfftb%nr3x

       IF ( dfftb%nr1 > dfftp%nr1 .or. dfftb%nr2 > dfftp%nr2 .or. dfftb%nr3 > dfftp%nr3 ) &
          CALL errore(' smallbox_grid_init ', ' box grid larger than dense grid?',1)
       RETURN

     END SUBROUTINE smallbox_grid_init

     SUBROUTINE smallbox_grid_info( dfftb )
       !
       USE io_global,  ONLY: stdout, ionode
       USE fft_types,  ONLY: fft_dlay_descriptor
       !
       TYPE(fft_dlay_descriptor), INTENT(IN) :: dfftb
       !
       IF ( ionode ) THEN
         IF ( dfftb%nr1 > 0 .AND. dfftb%nr2 > 0 .AND. dfftb%nr3 > 0 ) THEN
           WRITE( stdout,*)
           WRITE( stdout,*) '  Small Box Real Mesh'
           WRITE( stdout,*) '  -------------------'
           WRITE( stdout,1000) dfftb%nr1, dfftb%nr2, dfftb%nr3, dfftb%nr1, dfftb%nr2, dfftb%nr3, 1, 1, 1
           WRITE( stdout,1010) dfftb%nr1x, dfftb%nr2x, dfftb%nr3x
           WRITE( stdout,1020) dfftb%nnr
         END IF
       END IF

1000  FORMAT(3X, &
         'Global Dimensions   Local  Dimensions   Processor Grid',/,3X, &
         '.X.   .Y.   .Z.     .X.   .Y.   .Z.     .X.   .Y.   .Z.',/, &
         3(1X,I5),2X,3(1X,I5),2X,3(1X,I5) )
1010  FORMAT(3X, 'Array leading dimensions ( nr1x, nr2x, nr3x )   = ', 3(1X,I5) )
1020  FORMAT(3X, 'Local number of cell to store the grid ( nrxx ) = ', 1X, I9 )


     END SUBROUTINE smallbox_grid_info

!=----------------------------------------------------------------------------=!
   END MODULE smallbox_grid_dim
!=----------------------------------------------------------------------------=!
