/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001-2003 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!----------------------------------------------------------------------------
!
MODULE noncollin_module
  USE kinds, ONLY : DP
  USE parameters, ONLY : ntypx
  !
  SAVE
  !
  INTEGER :: &
      npol,               & !  number of coordinates of wfc
      report,             & !  print the local quantities (magnet. and rho)
                            !  every #report iterations
      nspin_lsda = 1,     & !  =1 when nspin=1,4 =2 when nspin=2 
      nspin_mag = 1,      & !  =1 when nspin=1,4 (domag=.false.), =2 when
                            !   nspin=2, =4 nspin=4 (domag=.true.)
      nspin_gga = 1,      & !  =1 when nspin=1,4 (domag=.false.)   
                            !  =2 when nspin=2,4 (domag=.true.) (needed with gga)
      i_cons = 0            !  indicator for constrained local quantities
  !
  INTEGER, ALLOCATABLE :: &
  !                         !  when spherical (non-overlapping) integration
      pointlist(:)          !  regions are defined around atoms this index
                            !  say for each point in the fft grid to which 
                            !  atom it is assigned (0 if no atom is selected)
  !
  LOGICAL :: &
      noncolin, &           !  true if noncollinear magnetism is allowed
      lsign=.FALSE.         !  if true use the sign feature to calculate
                            !  rhoup and rhodw
  !
  REAL (DP) :: &
      angle1(ntypx),       &!  Define the polar coordinates of the starting
      angle2(ntypx),       &!  magnetization's direction for each atom
      mcons(3,ntypx)=0.d0, &!  constrained values for local variables
      magtot_nc(3),        &!  total magnetization
      bfield(3)=0.d0,      &!  magnetic field used in some cases
      vtcon,               &!  contribution of the constraining fields to
                            !  the total energy
      r_m(ntypx) = 0.0d0,  &!  Radius for local integrations for each type
      lambda                !  prefactor in the penalty functional 
                            !  for constraints
  !
  REAL (DP), ALLOCATABLE :: &
      factlist(:),         &! weight factors for local integrations
      r_loc(:),            &! local integrated charge 
      m_loc(:,:)            ! local integrated magnetization

  REAL(DP) ::     &
     ux(3)                 ! versor for deciding signs in gga
  !
  CONTAINS
    !
    !------------------------------------------------------------------------
    SUBROUTINE deallocate_noncol()
      !------------------------------------------------------------------------
      !
      IF ( ALLOCATED( pointlist) )       DEALLOCATE( pointlist )
      IF ( ALLOCATED( factlist ) )       DEALLOCATE( factlist )
      IF ( ALLOCATED( r_loc    ) )       DEALLOCATE( r_loc )
      IF ( ALLOCATED( m_loc    ) )       DEALLOCATE( m_loc )
      !
    END SUBROUTINE deallocate_noncol
    !
END MODULE noncollin_module
