/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2010-2013 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!----------------------------------------------------------------------------
SUBROUTINE plugin_arguments()
  !-----------------------------------------------------------------------------
  !
  ! check for presence of command-line option "-plugin_name" or "--plugin_name"
  ! where "plugin_name" has to be set here. If such option is found, variable
  ! "use_plugin_name" is set and usage of "plugin_name" is thus enabled.
  ! Currently implemented: "plumed", "pw2casino" (both case-sensitive)
  !
  USE kinds,         ONLY : DP
  !
  USE io_global,     ONLY : stdout
  !
  USE plugin_flags
  !
  !
  IMPLICIT NONE
  !
  INTEGER  :: iiarg, nargs, iargc, i, i0
  CHARACTER (len=1), EXTERNAL ::  lowercase
  CHARACTER (len=256) :: arg
  !
  !
  !
  nargs = iargc()
  ! add here more plugins
  use_plumed = .false.
  use_pw2casino = .false.
  use_environ = .false.
  !
  DO iiarg = 1, nargs 
    CALL getarg( iiarg, plugin_name)
    IF ( plugin_name(1:1) == '-') THEN
       i0 = 1
       IF ( plugin_name(2:2) == '-') i0 = 2
       arg = ' '
       DO i=i0+1, LEN_TRIM (plugin_name)
          arg(i-i0:i-i0) = lowercase (plugin_name(i:i))
       END DO
!       write(0,*) "plugin_name: ", trim(arg)
       ! add here more plugins
       IF ( TRIM(arg)=='plumed' ) THEN
          use_plumed = .true.
       END IF
       IF ( TRIM(arg)=='pw2casino' ) THEN
          use_pw2casino = .true.
       ENDIF
       IF ( TRIM(arg)=='environ' ) THEN
          use_environ = .true.
       ENDIF
    ENDIF
  ENDDO
  !
  RETURN
  !
END SUBROUTINE plugin_arguments
!
!----------------------------------------------------------------------------
  SUBROUTINE plugin_arguments_bcast(root,comm)
  !----------------------------------------------------------------------------
  !
  ! broadcast plugin arguments
  !
  USE mp, ONLY : mp_bcast
  USE plugin_flags
  !
  IMPLICIT NONE
  !
  integer :: root
  integer :: comm
  !
  CALL mp_bcast(use_plumed,root,comm)
  !
  CALL mp_bcast(use_pw2casino,root,comm)
  !
  CALL mp_bcast(use_environ,root,comm)
  !
!  write(0,*) "use_plumed: ", use_plumed
  !
  RETURN
  !
END SUBROUTINE plugin_arguments_bcast

