/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2004-2007 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
! This module is USEd, for the time being, as an interface
! between the UPF pseudo type and the pseudo variables internal representation

!=----------------------------------------------------------------------------=!
  MODULE upf_to_internal
!=----------------------------------------------------------------------------=!

  IMPLICIT NONE
  PRIVATE
  PUBLIC :: set_pseudo_upf
  SAVE

!=----------------------------------------------------------------------------=!
  CONTAINS
!=----------------------------------------------------------------------------=!
!
!---------------------------------------------------------------------
subroutine set_pseudo_upf (is, upf, grid)
  !---------------------------------------------------------------------
  !
  !   set "is"-th pseudopotential using the Unified Pseudopotential Format
  !   "upf" - convert and copy to internal variables
  !   If "grid" is present, reconstruct radial grid.
  !   Obsolescent - for old-style PP formats only.
  !
  USE funct, ONLY: set_dft_from_name, set_dft_from_indices
  !
  USE pseudo_types
  USE radial_grids, ONLY: radial_grid_type, allocate_radial_grid
  !
  implicit none
  !
  INTEGER :: is
  TYPE (pseudo_upf) :: upf
  TYPE (radial_grid_type), target, optional :: grid
  !
  !     Local variables
  !
  integer :: iexch,icorr,igcx,igcc
  !
  ! old formats never contain "1/r" pseudopotentials
  !
  upf%tcoulombp = .false.
  !
  ! workaround for rrkj format - it contains the indices, not the name
  !
  if ( upf%dft(1:6)=='INDEX:') then
     read( upf%dft(7:10), '(4i1)') iexch,icorr,igcx,igcc
     call set_dft_from_indices(iexch,icorr,igcx,igcc, 0) !Cannot read nonloc in this format
  else
     call set_dft_from_name( upf%dft )
  end if
  !
  if(present(grid)) then
    call allocate_radial_grid(grid,upf%mesh)
    grid%dx   = upf%dx
    grid%xmin = upf%xmin
    grid%zmesh= upf%zmesh
    grid%mesh = upf%mesh
    !
    grid%r  (1:upf%mesh) = upf%r  (1:upf%mesh)
    grid%rab(1:upf%mesh) = upf%rab(1:upf%mesh)
    upf%grid => grid
  endif
  !
end subroutine set_pseudo_upf

!=----------------------------------------------------------------------------=!
  END MODULE upf_to_internal
!=----------------------------------------------------------------------------=!
