/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

! Copyright (C) 2006-2008 Dmitry Korotin - dmitry@korotin.name
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!--------------------------------------------------------------------------
!

MODULE wannier_new
  !
  ! ... Variables to construct and store wannier functions
  !
  USE kinds,      ONLY : DP
  !
  SAVE
  ! 
  INTEGER, PARAMETER :: ningx = 10 ! max number of trial wavefunction ingredients

  LOGICAL :: &
     use_wannier,              &! if .TRUE. wannier functions are constructed
     rkmesh,                   &! if .TRUE. regular k-mesh without symmetry is used !now used in input_parameters_mod
     plot_wannier,             &! if .TRUE. wannier number plot_wan_num is plotted
     use_energy_int,           &! if .TRUE. uses energy interval for wannier generation, not band numbers   
        print_wannier_coeff           ! if .TRUE. computes and prints coefficients of wannier decomp. on atomic functions
  INTEGER :: &
     nwan,                     &! number of wannier functions
     plot_wan_num,             &! number of wannier for plotting  
     plot_wan_spin              ! spin of wannier for plotting
  REAL(kind=DP), allocatable ::  &
     wan_pot(:,:),             &! constrained potential
     wannier_energy(:,:),      &! energy of each wannier (of each spin)
     wannier_occ(:,:,:)         ! occupation matrix of wannier functions(of each spin)
  COMPLEX(kind=DP), allocatable :: &
     pp(:,:),                  &! <phi|S|psi> projections
     coef(:,:,:)                ! coefficients of wannier decomp. on atomic functions
        
  TYPE ingredient
         INTEGER :: l = 0, &           ! l value for atomic wfc
                    m = 0, &           ! m value for atomic wfc
                    iatomwfc = 0       ! number of corresponding atomic orbital
         REAL :: c = 0.d0              ! coefficient
  END TYPE ingredient
  
  TYPE wannier_data
          INTEGER :: iatom = 0,              &
                     ning = 0
          REAL ::        bands_from = 0.d0, &
                         bands_to = 0.d0
          TYPE (ingredient) :: ing(ningx)
  END TYPE wannier_data
  
  TYPE (wannier_data), allocatable :: wan_in(:,:)
END MODULE wannier_new
