/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001-2008 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!-----------------------------------------------------------------------
subroutine bcast_d3_input
  !-----------------------------------------------------------------------
  !
  !     In this routine the first processor sends input data to all
  !     the other processors
  !
  !
  use pwcom
  use phcom
  use d3com
  use mp, only: mp_bcast
  use mp_world, only: world_comm
  use io_files, only: prefix, tmp_dir
  use ions_base, only: amass
  use control_flags, only: iverbosity
  use run_info, only: title

  implicit none
  integer :: root = 0
  !
  ! logicals
  !
  call mp_bcast (lgamma, root, world_comm)
  call mp_bcast (wraux, root, world_comm)
  call mp_bcast (recv, root, world_comm)
  call mp_bcast (testflag,root, world_comm)
  !
  ! integers
  !
  call mp_bcast (iverbosity, root, world_comm)
  call mp_bcast (testint, root, world_comm)
  call mp_bcast (q0mode_todo, root, world_comm)
  call mp_bcast (istop, root, world_comm)
  !
  ! real*8
  !
  call mp_bcast (amass, root, world_comm)
  call mp_bcast (xq, root, world_comm)
  call mp_bcast (ethr_ph, root, world_comm)
  call mp_bcast (testreal, root, world_comm)
  !
  ! characters
  !
  call mp_bcast (title, root, world_comm)
  call mp_bcast (fildyn, root, world_comm)
  call mp_bcast (fildrho, root, world_comm)
  call mp_bcast (fild0rho, root, world_comm)
  call mp_bcast (tmp_dir, root, world_comm)
  call mp_bcast (prefix, root, world_comm)
  return
end subroutine bcast_d3_input
