/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001-2003 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
SUBROUTINE d3_recover (ilab, isw)
  !-----------------------------------------------------------------------
  !
  !  isw = +1 Writes d3dyn in a file for possible recover
  !  isw = -1 Starts a recover run
  !
  USE pwcom
  USE phcom
  USE d3com
  USE io_global, ONLY : ionode
  USE mp,        ONLY: mp_bcast
  USE mp_world,  ONLY: world_comm
  USE io_files,  ONLY : seqopn
  !
  IMPLICIT NONE
  !
  INTEGER :: ilab, isw
  INTEGER :: root = 0
  LOGICAL :: exst

  iunrec = 98
  IF (isw.EQ.1) THEN
     !
     IF ( .NOT. ionode ) RETURN

     CALL seqopn (iunrec, 'recv_d3', 'unformatted', exst)
     IF (ilab.LE.4) THEN
        WRITE (iunrec) ilab
     ELSE
        WRITE (iunrec) ilab, d3dyn

     ENDIF

     CLOSE (unit = iunrec, status = 'keep')
  ELSEIF (isw.EQ. - 1) THEN
     !
     IF ( ionode ) THEN
        !
        CALL seqopn (iunrec, 'recv_d3', 'unformatted', exst)
        READ (iunrec) ilab
        IF (ilab.GE.5) THEN
           REWIND (iunrec)
           READ (iunrec) ilab, d3dyn

        ENDIF
        !
        CLOSE (unit = iunrec, status = 'keep')
        !
     END IF
     !
     CALL mp_bcast (d3dyn, root, world_comm)
     CALL mp_bcast (ilab, root, world_comm)
     !
  ENDIF
  RETURN
END SUBROUTINE d3_recover
