/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
subroutine drhod2v
  !-----------------------------------------------------------------------
  ! It calls the routines which calculate the term containing the first
  ! variation of the charge and the secon variation of the potential with
  ! respect to the perturbation.
  ! d0rhod2v: contains the terms depending on the first variation of the c
  ! with respect to a perturbaation at q=0
  ! dqrhod2v: contains the terms depending on the first variation of the c
  ! with respect to a perturbaation at a generic q
  ! The variation of the charge can be read from a file or calculated dire
  ! --this last option is to be used for testing pourposes--
  !
  USE ions_base,  ONLY : nat
  USE kinds, only : DP
  USE fft_base, ONLY : dfftp
  use pwcom
  use phcom
  use d3com
  !
  implicit none
  integer :: irr, irr1, imode0, ipert, ir
  real (DP) :: xq0 (3)
  complex (DP), allocatable :: drhoscf (:)
  ! the change of density due to perturbations

  allocate  (drhoscf( dfftp%nnr))

  call read_ef
  if (.not.allmodes) then
     do ipert = 1, 3 * nat
        call davcio_drho (drhoscf, lrdrho, iudrho, ipert, - 1)
        call dqrhod2v (ipert, drhoscf)
     enddo

  endif
  do ipert = 1, 3 * nat
     if (q0mode (ipert) ) then
        call davcio_drho (drhoscf, lrdrho, iud0rho, ipert, - 1)
        call d0rhod2v (ipert, drhoscf)
     endif

  enddo

  deallocate (drhoscf)
  return

end subroutine drhod2v
