/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!-----------------------------------------------------------------------
subroutine rotate_and_add_d3 (phi, phi2, nat, isym, s, invs, irt, &
     rtau, sxq)
!-----------------------------------------------------------------------
!  Rotates a third order matrix (phi) in crystal coordinates according
!  to the specified symmetry operation and add the rotated matrix
!  to phi2.   phi is left unmodified.
!
  USE kinds, ONLY : DP
  USE constants, ONLY : tpi
  implicit none
  !
  ! input variables
  !

  integer :: nat, isym, s (3, 3, 48), invs (48), irt (48, nat)
  ! number of atoms in the unit cell
  ! index of the symm.op.
  ! the symmetry operations
  ! index of the inverse operations
  ! index of the rotated atom

  complex (DP) :: phi (3, 3, 3, nat, nat, nat), phi2 (3, 3, 3, nat, nat, nat)
  ! the input d3dyn.mat.
  ! in crystal coordinates
  ! the rotated d3dyn.mat
  ! in crystal coordinates
  real (DP) :: rtau (3, 48, nat), sxq (3)
  ! for each atom and rotation gives
  ! the R vector involved
  ! the rotated q involved in this sym.op
  !
  !  local variables
  !

  integer :: na, nb, nc, sna, snb, snc, ism1, i, j, k, l, m, n
  ! counters on atoms
  ! indices of rotated atoms
  ! index of the inverse symm.op.
  ! generic counters

  real (DP) :: arg
  ! argument of the phase

  complex (DP) :: phase, work


  ism1 = invs(isym)
  do nc = 1, nat
     snc = irt(isym,nc)
     do na = 1, nat
        do nb = 1, nat
           sna = irt(isym,na)
           snb = irt(isym,nb)
           arg = (sxq (1) * (rtau(1,isym,na) - rtau(1,isym,nb) ) &
                + sxq (2) * (rtau(2,isym,na) - rtau(2,isym,nb) ) &
                + sxq (3) * (rtau(3,isym,na) - rtau(3,isym,nb) ) ) * tpi
           phase = CMPLX(cos(arg),-sin(arg),kind=DP)
           do m = 1, 3
              do i = 1, 3
                 do j = 1, 3
                    work = CMPLX(0.d0, 0.d0,kind=DP)
                    do k = 1, 3
                       do l = 1, 3
                          do n = 1, 3
                             work = work &
                                  + s(m,n,ism1) * s(i,k,ism1) * s(j,l,ism1) &
                                  * phi(n,k,l,nc,na,nb) * phase
                          enddo
                       enddo
                    enddo
                    phi2(m,i,j,snc,sna,snb) = phi2(m,i,j,snc,sna,snb) + work
                 enddo
              enddo

           enddo
        enddo
     enddo

  enddo
  return
end subroutine rotate_and_add_d3
