/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
subroutine set_d3irr
  !-----------------------------------------------------------------------
  !
  ! It computes a basis for all the irreducible representations of the
  ! group of the crystal, which are contained in the representation
  ! which has as basis the displacement vectors.
  ! This basis will be used for those quantities that depend on the
  ! q=0 perturbation.
  !
  ! Receives in input:   nsymg0, s, invs, irt, rtau
  ! Calculates:          ug0, tg0, npertg0, nirrg0, irgq
  !
  ! NB: It assumes that the phonon calculation for the q=0 case, has been
  ! performed with iswitch=-2 (modenum=0). If this is not the case the following
  ! routine does not work.
  !
  USE ions_base,  ONLY : nat
  USE kinds, only : DP
  use pwcom
  use symm_base, only : s, irt, invs
  USE control_flags, ONLY : iverbosity
  use phcom
  use d3com
  use io_files, only: tmp_dir

  implicit none
  integer :: w_nsymq, w_irotmq
  ! work array
  ! work array

  real (DP) :: zero (3), w_gi (3, 48), w_gimq (3), xqck(3)
  ! a null vector
  ! work array
  complex (DP) :: w_tmq (npertx, npertx, 3 * nat)
  ! work array

  logical :: w_minus_q
  ! work array

  zero = 0.0_dp

  w_minus_q = .true.
  if (nsymg0.gt.1) then
     !call io_pattern(nat,fild0rho,nirrg0,npertg0,ug0,xqck,tmp_dir,-1)
     call set_sym_irr (nat, at, bg, zero, s, invs, nsymg0, rtau, irt, &
          irgq, w_nsymq, w_minus_q, w_irotmq, tg0, w_tmq, npertx, &
          ug0, npertg0, nirrg0, w_gi, w_gimq, iverbosity)
  else
     call set_irr_nosym (nat, at, bg, zero, s, invs, nsymg0, rtau, &
          irt, irgq, w_nsymq, w_minus_q, w_irotmq, tg0, w_tmq,     &
          npertx, ug0, npertg0, nirrg0, w_gi, w_gimq, iverbosity)
  endif

  return
end subroutine set_d3irr

