/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
subroutine trntnsc_3 (phi, at, bg, iflg)
  !-----------------------------------------------------------------------
  !
  ! trasforms a COMPLEX third order tensor
  !(like the derivative of the dynamical matrix)
  ! from crystal to cartesian axis (iflg >=  1) or viceversa (iflg <= -1)
  !
  USE kinds, only : DP
  implicit none

  integer :: iflg
  ! input: gives the versus of the trans.

  complex (DP) :: phi (3, 3, 3)
  ! inp/out: the matrix to transform

  real (DP) :: at (3, 3), bg (3, 3)
  ! input: the direct lattice vectors
  ! input: the reciprocal lattice

  integer :: i, j, k, l, m, n
  !
  !  counters on polarizations
  !
  complex (DP) :: wrk (3, 3, 3)
  ! a work array

  if (iflg.gt.0) then
     !
     ! forward transformation (crystal to cartesian axis)
     !

     call zcopy (27, phi, 1, wrk, 1)
     do m = 1, 3
        do i = 1, 3
           do j = 1, 3
              phi (m, i, j) = (0.d0, 0.d0)
              do n = 1, 3
                 do k = 1, 3
                    do l = 1, 3
                       phi (m, i, j) = phi (m, i, j) + wrk (n, k, l) * bg (i, k) &
                            * bg (j, l) * bg (m, n)
                    enddo
                 enddo
              enddo
           enddo
        enddo
     enddo
  else
     !
     ! backward transformation (cartesian to crystal axis)
     !
     do m = 1, 3
        do i = 1, 3
           do j = 1, 3
              wrk (m, i, j) = (0.d0, 0.d0)
              do n = 1, 3
                 do k = 1, 3
                    do l = 1, 3
                       wrk (m, i, j) = wrk (m, i, j) + phi (n, k, l) * at (k, i) &
                            * at (l, j) * at (n, m)
                    enddo
                 enddo
              enddo
           enddo
        enddo
     enddo
     call zcopy (27, wrk, 1, phi, 1)
  endif
  return


end subroutine trntnsc_3
