/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
function w_1gauss (x, n)
!-----------------------------------------------------------------------
!
!     the derivative of w0gauss:
!
! --> (n=-99): second derivative of Fermi-Dirac function
!
USE kinds, ONLY : DP
USE constants, ONLY : sqrtpm1
!
implicit none
real (DP) :: w_1gauss, x
                       ! output: the value of the function
                      ! input: the point where to compute the function

integer :: n
                      ! input: the order of the smearing function
!
!    here the local variables
!
real (DP) :: a, arg, hp, hd, aux1, aux2
                      ! the coefficients a_n
                      ! the argument of the exponential
                      ! the hermite function
                      ! the hermite function
                      ! auxiliary variable
                      ! auxiliary variable
integer :: i, ni
                      ! counter on n values
                      ! counter on 2n values
! Fermi-Dirac smearing
if (n.eq. - 99) then
   aux1 = exp (x)
   aux2 = exp ( - x)
   w_1gauss = (aux2 - aux1) / (2.d0 + aux1 + aux2) **2
   return
endif
!
arg = min (200.d0, x**2)
w_1gauss = - 2.d0 * x * exp ( - arg) * sqrtpm1
if (n.eq.0) return
hd = exp ( - arg)
hp = 2.d0 * x * exp ( - arg)
ni = 1
a = sqrtpm1
do i = 1, n
hd = 2.0d0 * x * hp - 2.0d0 * DBLE (ni) * hd
ni = ni + 1
a = - a / (DBLE (i) * 4.0d0)
hp = 2.0d0 * x * hd-2.0d0 * DBLE (ni) * hp
ni = ni + 1
w_1gauss = w_1gauss - a * hp
enddo
return



end function w_1gauss
