/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
subroutine write_d3dyn (xq, phi, nat, iudyn, wrmode)
  !-----------------------------------------------------------------------
  !
  USE kinds, only : DP
  implicit none
  !
  ! input variables
  !
  integer :: iudyn, nat
  ! unit number
  ! number of atom in the unit cell
  complex (DP) :: phi (3, 3, 3, nat, nat, nat)
  !  derivative of the dynamical matrix
  real (DP) :: xq (3)
  ! the q vector
  logical :: wrmode (3 * nat)
  ! if .true. this mode is to be written
  !
  ! local variables
  !
  integer :: na, nb, nc, icar, jcar, kcar, i
  ! counters on atoms
  ! cartesian coordinate counters
  ! generic counter
  write (iudyn, 9000) (xq (icar), icar = 1, 3)
  do i = 1, 3 * nat
     if (wrmode (i) ) then
        write (iudyn, '(/,12x,"modo:",i5,/)') i
        nc = (i - 1) / 3 + 1
        kcar = i - 3 * (nc - 1)
        do na = 1, nat
           do nb = 1, nat
              write (iudyn, '(2i3)') na, nb
              do icar = 1, 3
                 write (iudyn, '(3e24.12)') (phi (kcar, icar, jcar, nc, na, nb) &
                      , jcar = 1, 3)
              enddo
           enddo
        enddo
     endif

  enddo

  return
9000 format(/,5x,'Third derivative in cartesian axes', &
       &       //,5x,'q = ( ',3f14.9,' ) ',/)
end subroutine write_d3dyn
