/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2003 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!----------------------------------------------------------------------------
!
! ... These are PHONON-specific modules (Conjugate Gradient version)
!
MODULE phunits
  !
  SAVE
  !
  CHARACTER(len=256) :: &
      fildyn
  CHARACTER(len=75) :: &
      title_ph
  INTEGER :: &
      iuwfc,     &!
      iubar,     &!
      iudwf,     &!
      iuscf,     &!
      iuvkb,     &!
      lrwfc,     &!
      lrbar,     &!
      lrdwf,     &!
      lrscf       !
  !
END MODULE phunits
!
!
MODULE flags
  !
  SAVE
  !
  LOGICAL :: &
      trans,     &!
      epsil,     &!
      raman,     &!
      equil,     &!
      nlcc_any,  &!
      asr,       &!
      recover
  !
END MODULE flags
!
!
MODULE dielectric
  USE kinds, ONLY :  DP
  !
  SAVE
  !
  REAL(DP) :: &
      epsilon0(3,3)
  REAL(DP), ALLOCATABLE :: &
      zstar(:,:,:)
  !
END MODULE dielectric
!
!
MODULE modes1
  USE kinds, ONLY :  DP
  !
  SAVE
  !
  INTEGER :: &
      nmodes
  REAL(DP), ALLOCATABLE :: &
      dyn(:,:),   &!
      u(:,:)       !
  !
END MODULE modes1
!
!
MODULE cgconv
  USE kinds, ONLY :  DP
  !
  SAVE
  !
  INTEGER :: &
      niter_ph
  !
  REAL(DP) :: tr2_ph
  !
END MODULE cgconv
!
!
MODULE AA
  USE kinds, ONLY :  DP
  !
  SAVE
  !
  COMPLEX(DP), ALLOCATABLE, TARGET :: &
       aux2(:),   &!
       aux3(:)     !
  REAL(DP), ALLOCATABLE, TARGET :: &
       auxr(:)     !
  !
END MODULE AA
!
!
MODULE dmu
  USE kinds, ONLY :  DP
  !
  SAVE
  !
  REAL(DP), ALLOCATABLE:: &
       dmuxc(:),        &!  d V_xc / d rho
       grho(:,:,:),     &!  gradient of the unperturbed density
       dvxc_rr(:,:,:),  &!
       dvxc_sr(:,:,:),  &!  derivatives of the E_xc functional w.r.t.
       dvxc_ss(:,:,:),  &!  r=rho and s=|grad(rho)|
       dvxc_s (:,:,:)
  !
END MODULE dmu
!
!
MODULE phon
  USE kinds, ONLY :  DP
  !
  SAVE
  !
  COMPLEX(DP), ALLOCATABLE:: &
       dvpsi(:,:),      &!
       dpsi(:,:)         !
  !
END MODULE phon
!
!
MODULE symmetry
  !
  SAVE
  !
  INTEGER :: &
       n_diff_sites,      &!
       nasr
  INTEGER, ALLOCATABLE :: &
       equiv_atoms(:,:),  &!
       n_equiv_atoms(:)    !
  INTEGER, ALLOCATABLE :: &
       has_equivalent(:)   !
  !
END MODULE symmetry
!
!
MODULE diffs
  USE kinds, ONLY :  DP
  !
  SAVE
  !
  INTEGER :: &
       nderiv,     &!
       first,      &!
       last         !
  REAL(DP) :: &
       deltatau     !
  !
END MODULE diffs
!
!
MODULE cgcom
  USE cgconv
  USE phunits
  USE flags
  USE modes1
  USE AA
  USE phon
  USE diffs
  USE dmu
  USE symmetry
  USE dielectric
END MODULE cgcom
