/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2003 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!---------------------------------------------------------------------
SUBROUTINE dvb_cc (nlcc,npseu,ngm,nrxx,  &
     nl,igtongl,rho_core,dmuxc,ga,aux,dvb_nlcc)
  !---------------------------------------------------------------------
  ! calculate the core-correction contribution to Delta V bare
  !
  USE kinds, ONLY : dp
  USE fft_base, ONLY : dffts, dfftp
  USE fft_interfaces, ONLY : fwfft, invfft
  IMPLICIT NONE
  INTEGER:: npseu,ngm,nrxx,np,ng,i
  LOGICAL :: nlcc(npseu)
  INTEGER :: nl(ngm), igtongl(ngm)
  real(dp) :: rho_core(*), dmuxc(nrxx)
  COMPLEX(dp) :: ga(ngm), dvb_nlcc(ngm), aux(nrxx)
  !
  DO np=1,npseu
     IF(nlcc(np)) GOTO 10
  ENDDO
  RETURN
10 CONTINUE
  !
  aux(:) = (0.d0, 0.d0)
  DO ng=1,ngm
     aux(nl(ng)) = ga(ng) * rho_core(igtongl(ng))
  ENDDO
  CALL invfft ('Dense', aux, dfftp)
  !
  aux(:) = aux(:) * dmuxc(:)
  !
  CALL fwfft ('Dense', aux, dfftp)
  DO ng=1,ngm
     dvb_nlcc(ng) = aux(nl(ng))
  ENDDO
  !
  RETURN
END SUBROUTINE dvb_cc

