/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2003-2010 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
SUBROUTINE generate_effective_charges (nat, nsym, s, invs, irt, at, bg, &
     n_diff_sites, equiv_atoms, has_equivalent, zstar)
  !-----------------------------------------------------------------------
  !
  ! generate all effective charges
  !
  USE kinds, ONLY : DP
  USE symme, ONLY : crys_to_cart, cart_to_crys
  IMPLICIT NONE
  INTEGER :: nat, nsym, n_diff_sites, irt(48,nat), equiv_atoms(nat,nat),&
       s(3,3,48), has_equivalent(nat), invs(48)
  INTEGER :: isym, na, ni, nj, sni, i, j, k, l
  real(DP) :: zstar(3,3,nat), at(3,3), bg(3,3)
  LOGICAL :: done(nat), no_equivalent_atoms
  !
  no_equivalent_atoms=.true.
  DO na = 1,nat
     no_equivalent_atoms = no_equivalent_atoms .and. has_equivalent(na)==0
  ENDDO
  IF (no_equivalent_atoms) RETURN
  ! transform to crystal axis
  DO na = 1,nat
     IF (has_equivalent(na)==0 ) THEN
        CALL cart_to_crys ( zstar(:,:,na) )
        done(na)=.true.
     ELSE
        zstar(:,:,na) = 0.d0
        done(na)=.false.
     ENDIF
  ENDDO
  !
  DO isym = 1,nsym
     DO na = 1,n_diff_sites
        ni = equiv_atoms(na,1)
        sni = irt(isym,ni)
        IF ( .not.done(sni) ) THEN
           DO i = 1,3
              DO j = 1,3
                 DO k = 1,3
                    DO l = 1,3
                       zstar(i,j,sni) =  zstar(i,j,sni) +  &
                            s(i,k,invs(isym))*s(j,l,invs(isym))*zstar(k,l,ni)
                    ENDDO
                 ENDDO
              ENDDO
           ENDDO
           done(sni)=.true.
        ENDIF
     ENDDO
  ENDDO
  ! back to cartesian axis
  DO na = 1,nat
     CALL crys_to_cart ( zstar(:,:,na) )
  ENDDO
  !
  RETURN
END SUBROUTINE generate_effective_charges
