/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001-2008 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!-----------------------------------------------------------------------
subroutine bcast_ph_input ( )
  !-----------------------------------------------------------------------
  !
  !     In this routine the first processor sends the phonon input to all
  !     the other processors
  !
  !

  use mp, only: mp_bcast
  use mp_world, only: world_comm
  USE control_ph, ONLY : start_irr, last_irr, start_q, last_q, nmix_ph, &
                         niter_ph, lnoloc, alpha_mix, tr2_ph, lrpa, recover, &
                         ldisp, reduce_io, zue, zeu, epsil, trans, &
                         lgamma, ldiag, lqdir, search_sym,  electron_phonon, &
                         qplot, only_init, only_wfc, low_directory_check
  USE gamma_gamma, ONLY : asr
  USE disp, ONLY : nq1, nq2, nq3
  USE partial, ONLY : nat_todo
  USE freq_ph, ONLY : fpol
  USE output, ONLY : fildvscf, fildyn, fildrho
  use io_files, ONLY : outdir, prefix
  USE control_flags, only: iverbosity, modenum
  USE ramanm, ONLY: lraman, elop, dek, eth_rps, eth_ns
  USE input_parameters, ONLY: max_seconds
  USE input_parameters, ONLY : nk1, nk2, nk3, k1, k2, k3
  USE ions_base,     ONLY : amass
  USE io_global,   ONLY : meta_ionode_id
  USE run_info,   ONLY : title
  USE el_phon,    ONLY : elph_nbnd_min,elph_nbnd_max,el_ph_ngauss, el_ph_nsigma, el_ph_sigma
  USE dfile_star, ONLY : drho_star, dvscf_star

  implicit none
  !
  ! logicals
  !
  call mp_bcast (lgamma, meta_ionode_id, world_comm )
  call mp_bcast (epsil, meta_ionode_id, world_comm )
  call mp_bcast (trans, meta_ionode_id, world_comm )
  call mp_bcast (zue, meta_ionode_id, world_comm )
  call mp_bcast (zeu, meta_ionode_id, world_comm )
  call mp_bcast (reduce_io, meta_ionode_id, world_comm )
  call mp_bcast (ldisp, meta_ionode_id, world_comm )
  call mp_bcast (lraman, meta_ionode_id, world_comm )
  call mp_bcast (elop, meta_ionode_id, world_comm )
  call mp_bcast (fpol, meta_ionode_id, world_comm )
  call mp_bcast (recover, meta_ionode_id, world_comm )
  call mp_bcast (asr, meta_ionode_id, world_comm )
  call mp_bcast (lrpa, meta_ionode_id, world_comm )
  call mp_bcast (lnoloc, meta_ionode_id, world_comm )
  call mp_bcast (ldiag, meta_ionode_id, world_comm )
  call mp_bcast (lqdir, meta_ionode_id, world_comm )
  call mp_bcast (qplot, meta_ionode_id, world_comm )
  call mp_bcast (only_wfc, meta_ionode_id, world_comm )
  call mp_bcast (only_init, meta_ionode_id, world_comm )
  call mp_bcast (search_sym, meta_ionode_id, world_comm)
  !
  ! integers
  !
  call mp_bcast (start_irr, meta_ionode_id, world_comm )
  call mp_bcast (last_irr, meta_ionode_id, world_comm )
  call mp_bcast (start_q, meta_ionode_id, world_comm )
  call mp_bcast (last_q, meta_ionode_id, world_comm )
  call mp_bcast (niter_ph, meta_ionode_id, world_comm )
  call mp_bcast (nmix_ph, meta_ionode_id, world_comm )
  call mp_bcast (iverbosity, meta_ionode_id, world_comm )
  call mp_bcast (modenum, meta_ionode_id, world_comm )
  call mp_bcast (nat_todo, meta_ionode_id, world_comm )
  CALL mp_bcast( nq1, meta_ionode_id, world_comm )
  CALL mp_bcast( nq2, meta_ionode_id, world_comm )
  CALL mp_bcast( nq3, meta_ionode_id, world_comm )
  CALL mp_bcast( nk1, meta_ionode_id, world_comm )
  CALL mp_bcast( nk2, meta_ionode_id, world_comm )
  CALL mp_bcast( nk3, meta_ionode_id, world_comm )
  CALL mp_bcast( k1, meta_ionode_id, world_comm )
  CALL mp_bcast( k2, meta_ionode_id, world_comm )
  CALL mp_bcast( k3, meta_ionode_id, world_comm )
  CALL mp_bcast( low_directory_check, meta_ionode_id, world_comm )
  CALL mp_bcast( elph_nbnd_min, meta_ionode_id, world_comm )
  CALL mp_bcast( elph_nbnd_max, meta_ionode_id, world_comm )
  CALL mp_bcast( el_ph_ngauss, meta_ionode_id, world_comm )
  CALL mp_bcast( el_ph_nsigma, meta_ionode_id, world_comm )
  !
  ! real*8
  !
  call mp_bcast (tr2_ph, meta_ionode_id, world_comm )
  call mp_bcast (eth_rps, meta_ionode_id, world_comm )
  call mp_bcast (eth_ns, meta_ionode_id, world_comm )
  call mp_bcast (amass, meta_ionode_id, world_comm )
  call mp_bcast (alpha_mix, meta_ionode_id, world_comm )
  call mp_bcast (max_seconds, meta_ionode_id, world_comm )
  call mp_bcast (dek, meta_ionode_id, world_comm )
  CALL mp_bcast( el_ph_sigma, meta_ionode_id, world_comm )
  !
  ! characters
  !
  call mp_bcast (title, meta_ionode_id, world_comm )
  call mp_bcast (fildyn, meta_ionode_id, world_comm )
  call mp_bcast (fildvscf, meta_ionode_id, world_comm )
  call mp_bcast (fildrho, meta_ionode_id, world_comm )
  call mp_bcast (outdir, meta_ionode_id, world_comm )
  call mp_bcast (prefix, meta_ionode_id, world_comm )
  call mp_bcast (electron_phonon, meta_ionode_id, world_comm )
  !
  ! derived type (one bit at a time)
  !
  call mp_bcast (drho_star%open,  meta_ionode_id, world_comm)
  call mp_bcast (drho_star%pat,   meta_ionode_id, world_comm)
  call mp_bcast (drho_star%dir,   meta_ionode_id, world_comm)
  call mp_bcast (drho_star%ext,   meta_ionode_id, world_comm)
  call mp_bcast (drho_star%basis, meta_ionode_id, world_comm)
  !
  call mp_bcast (dvscf_star%open,  meta_ionode_id, world_comm)
  call mp_bcast (dvscf_star%pat,   meta_ionode_id, world_comm)
  call mp_bcast (dvscf_star%dir,   meta_ionode_id, world_comm)
  call mp_bcast (dvscf_star%ext,   meta_ionode_id, world_comm)
  call mp_bcast (dvscf_star%basis, meta_ionode_id, world_comm)

  return
end subroutine bcast_ph_input
