/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2010 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
LOGICAL FUNCTION check_q_points_sym(nqs, q, at, bg, nsym, s, invs, &
                                    nq1, nq2, nq3)
!
!  This function returns .true. if the mesh of q points given as input
!  is compatible with the FFT mesh. It returns .false. if a rotation of
!  the point group gives a q point that is not in the FFT mesh.
!
!
USE kinds, ONLY : DP
IMPLICIT NONE
INTEGER, INTENT(IN) :: nqs, nsym
INTEGER, INTENT(IN) :: nq1, nq2, nq3
INTEGER, INTENT(IN) :: s(3,3,48), invs(48)
REAL(DP), INTENT(IN) :: q(3,nqs), at(3,3), bg(3,3)

INTEGER :: nq, ipol, icar, iq, jq
INTEGER :: nr(3), isq (48), imq
LOGICAL :: lq
REAL(DP) :: xq, sxq(3,48)
REAL(DP) :: eps=1.d-5

nr(1)=nq1
nr(2)=nq2
nr(3)=nq3
lq = .TRUE.
DO iq = 1,nqs
   call star_q (q(:,iq), at, bg, nsym, s, invs, nq, sxq, isq, imq, .FALSE. )
   DO jq=1,nq
      DO ipol=1,3
         xq = 0.0d0
         DO icar=1,3
            xq = xq + at(icar,ipol) * sxq(icar,jq) * nr(ipol)
         END DO
         lq = lq .AND. (ABS(NINT(xq) - xq) .LT. eps)
      ENDDO
   ENDDO
ENDDO
check_q_points_sym=lq

RETURN
END FUNCTION check_q_points_sym
