/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001-2007 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!---------------------------------------------------------------------
subroutine compute_becalp (becq, alpq)
  !---------------------------------------------------------------------
  !
  !     This routine is used only at finite q and in this case
  !     computes the scalar product of vkb and psi_{k+q}, and of
  !     the derivative of vkb and psi_{k+q}. Eq. B8 and B10 (at k+q)
  !     of PRB 64 235118 (2001).
  !

  USE kinds, only : DP
  USE cell_base, ONLY : tpiba
  USE klist,     ONLY : xk
  USE gvect,     ONLY : g
  USE becmod, ONLY: calbec, bec_type, becscal
  USE buffers, ONLY: get_buffer
  USE uspp, ONLY: nkb, vkb
  USE noncollin_module, ONLY : noncolin, npol
  USE io_files, ONLY: iunigk
  USE wvfct,    ONLY : nbnd, npw, npwx, igk
  USE paw_variables, ONLY : okpaw

  USE control_ph, ONLY : lgamma, rec_code_read
  USE eqv, ONLY : evq
  USE units_ph, ONLY : lrwfc, iuwfc
  USE qpoint, ONLY : nksq, npwq, igkq, ikqs

  implicit none

  type (bec_type) :: becq(nksq), alpq(3,nksq)
  ! the becp with psi_{k+q}
  ! the alphap with psi_{k+q}

  integer :: ik, ikq, ipol, ibnd, ig, ios
  ! counter on k points
  ! counter on polarizations, bands and
  ! used for i/o control

  complex(DP) :: fact
  complex(DP), allocatable :: aux (:,:)
  !
  if (lgamma) return
  IF (rec_code_read >= -20.AND..NOT.okpaw) RETURN

  allocate (aux ( npwx*npol , nbnd))
  if (nksq.gt.1) rewind (iunigk)
  do ik = 1, nksq

     ikq = ikqs(ik)
     if (nksq.gt.1) then
        read (iunigk, err = 100, iostat = ios) npw, igk

100     call errore ('compute_becalp', 'reading igk', abs (ios) )
        read (iunigk, err = 200, iostat = ios) npwq, igkq
200     call errore ('compute_becalp', 'reading igkq', abs (ios) )

     endif
     call init_us_2 (npwq, igkq, xk (1, ikq), vkb)
     call get_buffer (evq, lrwfc, iuwfc, ikq)
     call calbec ( npwq, vkb, evq, becq(ik) )
     do ipol = 1, 3
        aux=(0.d0,0.d0)
        do ibnd = 1, nbnd
           do ig = 1, npwq
              aux (ig, ibnd) = evq (ig, ibnd) * &
                   (xk (ipol, ikq) + g (ipol, igkq(ig) ) )
           enddo
           IF (noncolin) THEN
              do ig = 1, npwq
                 aux (ig+npwx, ibnd) = evq (ig+npwx, ibnd) * &
                   (xk (ipol, ikq) + g (ipol, igkq(ig) ) )
              enddo
           ENDIF
        enddo

        call calbec ( npwq, vkb, aux, alpq(ipol,ik) )
     enddo
  enddo
  fact = CMPLX(0.d0, tpiba,kind=DP)

  DO ik=1,nksq
     DO ipol=1,3
        CALL becscal(fact,alpq(ipol,ik),nkb,nbnd)
     ENDDO
  ENDDO

  deallocate (aux)
  return
end subroutine compute_becalp
