/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2008-2012 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!

!-----------------------------------------------------------------------
SUBROUTINE find_irrep()
  !---------------------------------------------------------------------
  !
  !  Computes the variables needed to pass to the pattern representation
  !     u      the patterns
  !     nirr   the number of irreducible representation
  !     npert  the dimension of each irreducible representation
  !
  !
  USE kinds,         ONLY : DP
  USE ions_base,     ONLY : nat
  USE symm_base,     ONLY : nsym
  USE control_ph,    ONLY : lgamma_gamma
  USE modes,         ONLY : u, npert, nirr
  USE qpoint,        ONLY : xq
  USE control_flags, ONLY : modenum

  IMPLICIT NONE

  REAL(DP) :: w2(3*nat)

  IF (nsym > 1.AND..NOT.lgamma_gamma.AND.modenum==0) THEN
     CALL set_irr_new (xq, u, npert, nirr, w2)
  ELSE
     CALL set_irr_nosym_new (u, npert, nirr)
  ENDIF

  RETURN
  END SUBROUTINE find_irrep

!-----------------------------------------------------------------------
SUBROUTINE find_irrep_sym()
  !-----------------------------------------------------------------------
  !
  !   Computes the variables needed to symmetrize in the pattern representation
  !     t      the matrices of the small group of q on the pattern basis
  !     tmq    the matrix of the symmetry which sends q -> -q + G
  !
  !
  USE kinds,         ONLY : DP
  USE control_ph,    ONLY : lgamma_gamma
  USE symm_base,     ONLY : nsym
  USE modes,         ONLY : npertx, npert, nirr, t, tmq

  IMPLICIT NONE

  INTEGER :: irr
  ! counters

  IF (lgamma_gamma) RETURN

  npertx = 0
  DO irr = 1, nirr
     npertx = max (npertx, npert (irr) )
  ENDDO
  CALL allocate_pert()
  CALL set_irr_sym_new (t, tmq, npertx )

  RETURN
END SUBROUTINE find_irrep_sym
