/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2003-2010 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
subroutine generate_effective_charges_c &
     (nat, nsym, s, invs, irt, at, bg, n_diff_sites, equiv_atoms, &
      has_equivalent, asr, nasr, zv, ityp, ntyp, atm, zstar)
  !-----------------------------------------------------------------------
  !
  ! generate all effective charges
  !
  USE io_global, ONLY : stdout
  USE kinds, only : DP
  USE symme, only : crys_to_cart
  implicit none
  integer :: nat, nsym, n_diff_sites, irt(48,nat), equiv_atoms(nat,nat),&
       s(3,3,48), invs(48), has_equivalent(nat), nasr
  logical :: asr
  integer :: isym, na, ni, sni, i, j, k, l
  integer :: ityp(nat), ntyp
  real(DP) :: zstar(3,3,nat), at(3,3), bg(3,3), sumz, zv(ntyp)
  logical :: done(nat), no_equivalent_atoms
  character(3) :: atm(ntyp)
  !
  no_equivalent_atoms=.true.
  do na = 1,nat
     no_equivalent_atoms = no_equivalent_atoms .and. has_equivalent(na).eq.0
  end do
  if (no_equivalent_atoms) goto 100
  !
  !  zstar in input is in crystal axis
  !
  do na = 1,nat
     if (has_equivalent(na).eq.0 ) then
        done(na)=.true.
     else
        zstar(:,:,na) = 0.d0
        done(na)=.false.
     end if
  end do
  !
  do isym = 1,nsym
     do na = 1,n_diff_sites
        ni = equiv_atoms(na,1)
        sni = irt(isym,ni)
        if ( .not.done(sni) ) then
           do i = 1,3
              do j = 1,3
                 do k = 1,3
                    do l = 1,3
                       zstar(i,j,sni) =  zstar(i,j,sni) +  &
                            s(i,k,invs(isym))*s(j,l,invs(isym))*zstar(k,l,ni)
                    end do
                 end do
              end do
           end do
           done(sni)=.true.
        end if
     end do
  end do

100 continue
  !
  ! return to Cartesian axis
  !
  do na = 1,nat
     call crys_to_cart ( zstar(:,:,na) )
  end do
  !
  ! add the diagonal part
  !
  do i = 1, 3
     do na = 1, nat
        zstar(i, i, na) = zstar (i, i, na) + zv (ityp (na) )
     enddo
  enddo
  IF (asr.AND.nasr>0) THEN
     DO i=1,3
        DO j=1,3
           sumz=0.0_DP
           DO na=1,nat
              IF (na.ne.nasr) sumz=sumz+zstar(i,j,na)
           ENDDO
           zstar(i,j,nasr)=-sumz
        ENDDO
     ENDDO
  ENDIF
  !
  return
end subroutine generate_effective_charges_c
