/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!-----------------------------------------------------------------------
subroutine raman
  !-----------------------------------------------------------------------
  !
  USE kinds,    ONLY : DP
  USE klist,    ONLY : lgauss
  USE lsda_mod, ONLY : lsda
  USE control_flags, ONLY : gamma_only
  USE uspp, ONLY: okvan
  USE control_ph, ONLY : epsil, convt, rec_code_read, lgamma
  USE ph_restart, ONLY : ph_writefile
  USE ramanm, ONLY: lraman, elop, done_lraman, done_elop
  implicit none

  INTEGER :: ierr

  if (okvan) &
      call errore ('raman','Ultrasoft pseudopotentials not implemented',1)
  if (lsda) call errore ('raman',' spin-polarized case not implemented',1)
  if (lgauss .or..not.lgamma) &
      call errore ('raman','called in the wrong case',1)
  if (epsil.and..not.convt) &
      call errore ('raman','epsil calcul. not converged',1)
  !
  ! Computes Pc [DH,Drho] |psi>
  !
  IF (rec_code_read == -10) THEN
     ! restart from a previous calculation
     write (6,'(/,5x,''Skipping computation of Pc [DH,Drho] |psi> '')')
  ELSE
     write (6,'(/,5x,''Computing Pc [DH,Drho] |psi> '')')
     call dhdrhopsi ( ) 
  END IF
  !
  ! Computes the electro-optic tensor
  !
  IF (elop.AND..NOT.done_elop) THEN
     call el_opt()
  ELSEIF (done_elop) THEN
     CALL summarize_elopt()
  ENDIF
     
  if (.not.lraman) return
  write (6,'(/,5x,''Computing Second order response '')')
  !
  ! Computes the potential that remains unchanged in the scf-cycle
  !
  call dvpsi_e2 ( ) 
  !
  ! Self-consistent cycle to compute the second derivative of the charge
  !
  call solve_e2 ( )
  !
  ! Computes and writes the Raman tensor
  !
  call raman_mat ( )

  done_lraman=.TRUE.
  CALL ph_writefile('tensors',0, 0, ierr)
  return
end subroutine raman
