/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
MODULE ramanm
  !
  USE kinds,  ONLY : dp
  !
  SAVE
  !
  REAL (dp) :: eth_rps, eth_ns
  REAL (DP), ALLOCATABLE ::  ramtns (:,:,:,:)
  ! raman tensor (3, 3, 3, nat)
  REAL (DP) :: eloptns (3,3,3), dek
  ! electro-optic tensor
  ! delta_xk used for wavefunctions derivation

  LOGICAL :: lraman, elop, done_lraman=.FALSE., done_elop=.FALSE.
  INTEGER :: iuchf, iud2w, iuba2, lrchf, lrd2w, lrba2
  ! unit for chi-wavefuntion
  ! unit for the second derivative of the wavefunction
  ! unit for nscf part of the potential in scf calc.
  ! length of chi-wavefunction record
  ! length of iud2w record
  ! length of iuba2 record

  INTEGER ::  jab(3,3), a1j(6), a2j(6)
  ! 1-D index defining a 3x3 symmetric tensor
  ! a1j and a2j are the two components of the
  ! symmetric tensor defined by jab(3,3)
  !
  ! When dealing with a 3x3 symmetric tensor A2(i,j), the program considers a
  ! 6-components vector A1(k) whose components are defined by jab, a1j, a2j
  ! --common variables-- in the following way: A1(jab(i,j)) = A2(i,j);
  ! or A2(a1j(k),a2j(k)) = A2(a2j(k),a1j(k)) = A1(k)
  !
  data jab /1, 4, 6, 4, 2, 5, 6, 5, 3/
  data a1j /1, 2, 3, 1, 2, 1 /
  data a2j /1, 2, 3, 2, 3, 3 /
  !
END MODULE ramanm

