/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001-2008 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!-----------------------------------------------------------------------
subroutine rotate_dvscf_star(iq_)
  !-----------------------------------------------------------------------
  !
  !
  ! Given dvscf or drho, this routine obtains dvscf or drho
  ! over the star{q} and writes it on a file. 
  !

  USE kinds,         ONLY : DP
  USE cell_base,     ONLY : at, bg
  USE ions_base,     ONLY : ntyp => nsp, ityp
  USE symm_base,     ONLY : s, sr, irt, nsym, time_reversal, invs
  USE qpoint,        ONLY : xq
  USE output,        ONLY : fildrho, fildvscf
  USE dfile_star,    ONLY : write_dfile_star, drho_star, dvscf_star !write_dfile_mq
  USE units_ph,      ONLY : iudrho, iudvscf
  USE modes,         ONLY : u

  INTEGER :: nq,  isq (48), imq, iq_
  LOGICAL :: opnd
  REAL(DP) :: sxq (3, 48)

  if(.not.drho_star%open.and..not.dvscf_star%open) return

  call start_clock('rotate_dvscf_star')
  !
  !   Generates the star of q
  !
  call star_q (xq, at, bg, nsym, s, invs, nq, sxq, isq, imq, .TRUE. )
  !
  ! Rotates and write drho_q* (to be improved)
  IF(drho_star%open) THEN
     INQUIRE (UNIT = iudrho, OPENED = opnd)
     IF (opnd) CLOSE(UNIT = iudrho, STATUS='keep')
     CALL write_dfile_star(drho_star, fildrho, nsym, xq, u, nq, sxq, isq, &
          s, sr, invs, irt, ntyp, ityp,(imq==0), -1 )
  ENDIF
  IF(dvscf_star%open) THEN
     INQUIRE (UNIT = iudvscf, OPENED = opnd)
     IF (opnd) CLOSE(UNIT = iudvscf, STATUS='keep')
     CALL write_dfile_star(dvscf_star, fildvscf, nsym, xq, u, nq, sxq, isq, &
          s, sr, invs, irt, ntyp, ityp,(imq==0), iq_ )
  ENDIF
  !
  call stop_clock('rotate_dvscf_star')
  !
end subroutine rotate_dvscf_star
