/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2013 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
LOGICAL FUNCTION symmorphic_or_nzb()
!
! This function returns true if the small group of the current q is symmorphic
! or if the q point is not at zone border. Presently the routine that
! finds the mode symmetry works only when this function is .true..
!
  USE symm_base, ONLY : ftau
  USE modes,    ONLY : gi, nsymq
  IMPLICIT NONE
  LOGICAL :: is_symmorphic, result_sym
  INTEGER :: isym

  is_symmorphic=.NOT.(ANY(ftau(:,1:nsymq) /= 0))
  IF (is_symmorphic) THEN
     symmorphic_or_nzb=.TRUE.
     RETURN
  ELSE
     result_sym=.TRUE.
     DO isym=1,nsymq
        result_sym=( result_sym .AND. (abs(gi(1,isym))<1.d-8).and.  &
                                      (abs(gi(2,isym))<1.d-8).and.  &
                                      (abs(gi(3,isym))<1.d-8) )
     END DO
     symmorphic_or_nzb=result_sym
  END IF
  RETURN
END FUNCTION symmorphic_or_nzb
