/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2013 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
  SUBROUTINE write_qplot_data(auxdyn)
!
!  This routine writes on output the phonon frequencies in a format that
!  can be read by the program plotband. It is used only if the qplot
!  option is used in the phonon code. It writes in a file called fildyn.plot
!
  USE kinds,      ONLY : DP
  USE constants,  ONLY : ry_to_cmm1
  USE ions_base,  ONLY : nat
  USE disp,       ONLY : nqs, omega_disp, x_q, done_iq
  USE control_ph, ONLY : qplot
  USE el_phon,    ONLY : elph_simple, gamma_disp, el_ph_nsigma
  USE mp_images,  ONLY : nimage
  USE output,     ONLY : fildyn
  USE io_global,  ONLY : ionode

  IMPLICIT NONE
  CHARACTER(LEN=256), INTENT(IN) :: auxdyn
  CHARACTER(LEN=256) :: filename
  INTEGER, EXTERNAL :: find_free_unit
  CHARACTER(LEN=6) :: int_to_char
  REAL(DP) :: w1(3*nat)
  INTEGER :: iunit 
  INTEGER :: n, i, isig, iq

  IF ( .NOT. qplot ) CALL errore('write_qplot_data', &
                                 'called in the wrong case', 1)
!
!  Do not write if there are more than one image
!
  IF (nimage > 1) RETURN
!
!  This routine writes the files only when all q points have been done.
!
  DO iq=1,nqs
     IF (.NOT.done_iq(iq)) RETURN
  ENDDO

  IF (ionode) THEN
     iunit = find_free_unit()
     filename = TRIM(auxdyn) // '.freq'
     OPEN (unit=iunit,file=TRIM(filename),status='unknown',form='formatted')
     WRITE(iunit, '(" &plot nbnd=",i4,", nks=",i4," /")') 3*nat, nqs
     DO n=1, nqs
        WRITE(iunit,'(10x,3f10.6)')  x_q(1,n), x_q(2,n), x_q(3,n)
        DO i=1, 3*nat
           w1(i) = SQRT (ABS (omega_disp (i,n)) ) * ry_to_cmm1
           IF ( omega_disp(i,n) < 0.d0) w1(i) = - w1(i)
        ENDDO
        WRITE(iunit,'(6f10.4)') (w1(i), i=1,3*nat)
     END DO
     CLOSE(unit=iunit)
     IF (elph_simple) THEN
        DO isig=1, el_ph_nsigma
           filename = TRIM(auxdyn) // '.elph.'// int_to_char(isig)
           OPEN (unit=iunit,file=TRIM(filename),status='unknown', &
                                                form='formatted')
           WRITE(iunit, '(" &plot nbnd=",i4,", nks=",i4," /")') 3*nat, nqs
           DO n=1, nqs
              WRITE(iunit,'(10x,3f10.6)')  x_q(1,n), x_q(2,n), x_q(3,n)
              WRITE(iunit,'(6f10.4)') (gamma_disp(i,isig,n), i=1,3*nat)
           END DO
           CLOSE(unit=iunit)
        END DO
     ENDIF 
  ENDIF

  RETURN
  END SUBROUTINE write_qplot_data
