/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!-----------------------------------------------------------------------
subroutine write_ramtns (iudyn, ramtns)
  !-----------------------------------------------------------------------
  USE kinds, ONLY : DP
  USE constants, ONLY : fpi, BOHR_RADIUS_ANGS
  USE cell_base, ONLY : omega
  USE ions_base, ONLY : nat
  USE control_ph, ONLY : xmldyn
  !
  implicit none
  integer, intent(in) :: iudyn  ! unit
  real(DP), intent(in) :: ramtns(3, 3, 3, nat) ! raman tensor

  ! local variables
  integer :: na, ic, jc, kc
  ! counters
  real (DP), parameter ::   convfact = BOHR_RADIUS_ANGS**2
  ! conversion factor from au^2 to A^2
  !
  ! write raman tensor (D chi/d tau in A^2) to iudyn
  !
  IF (xmldyn) RETURN
  write(iudyn,'(/5x,"Raman tensor (A^2)",/)')
  do na = 1, nat
     do kc = 1, 3
        write (iudyn,'(5x,"atom # ",i4,"    pol.",i3)') na, kc
        write (iudyn, '(3e24.12)') ( (ramtns(ic, jc, kc, na) * &
              omega/fpi*convfact, ic = 1, 3), jc = 1, 3)
     enddo
  enddo

  return
end subroutine write_ramtns
