/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001-2003 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!--------------------------------------------------------------------
SUBROUTINE dos_g (et, nspin, nbnd, nks, wk, Degauss, ngauss, E, dosg)
  !--------------------------------------------------------------------
  !
  USE kinds, ONLY : DP
  IMPLICIT NONE
  INTEGER :: nspin, nks, nbnd, ngauss

  real(DP) :: wk (nks), et (nbnd, nks), Degauss, E, dosg (2)
  real(DP) :: w0gauss
  INTEGER :: n, ns, nk0, nk, ik
  INTEGER :: nspin0
  EXTERNAL w0gauss
  !
  IF (nspin == 1 .or. nspin == 4) THEN
     nk = nks
  ELSE
     nk = nks / 2
  ENDIF
  nspin0=nspin
  IF (nspin==4) nspin0=1
  !
  DO ns = 1, nspin0
     IF (ns==1) THEN
        nk0 = 1
     ELSE
        nk0 = nks / 2 + 1
     ENDIF
     dosg (ns) = 0.0d0
     DO ik = nk0, nk0 + nk-1
        DO n = 1, nbnd
           dosg (ns) = dosg (ns) + wk (ik) * w0gauss ( (E-et (n, ik) ) &
                / Degauss, ngauss)
        ENDDO
     ENDDO
     !
     dosg (ns) = dosg (ns) / Degauss
     !
  ENDDO
  !
  RETURN
END SUBROUTINE dos_g
