/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
SUBROUTINE ggen1d (ngm1d, g1d, gg1d, ig1dto3d, nl1d, igtongl1d)
  !
  !    This subroutine is the one dimensional equivalent of the ggen
  !    routine. It is used to produce the G vectors in the z directions
  !    to compute and output planar averages of the charge density
  !    of single levels. It should work in parallel machines.
  !
  USE kinds, ONLY: DP
  USE cell_base, ONLY : at
  USE gvect, ONLY: ngm, g, gg, igtongl
  USE fft_base, ONLY: dfftp
  IMPLICIT NONE
  INTEGER :: ngm1d, ig1dto3d (dfftp%nr3), igtongl1d (dfftp%nr3), nl1d (dfftp%nr3)
  ! output: the number of 1D G vectors on this processor
  ! output: correspondence 1D with 3D G vectors
  ! output: the correspondence with the shells
  ! output: correspondence 1D FFT mesh G with array

  real(DP) :: g1d (3, dfftp%nr3), gg1d (dfftp%nr3)
  ! output: ngm1d 3D vectors with the 1D G of this
  ! output: ngm1d scalars with the modulus of 1D G
  !
  !   local variables
  !

  INTEGER :: ig, ig1d
  ! counter on 3D vectors
  ! counter on 1D vectors

  real(DP), PARAMETER :: eps = 1.d-12


  g1d(:,:) = 0.d0
  gg1d(:) = 0.d0

  ig1d = 0
  DO ig = 1, ngm
     IF ( (abs(g(1,ig))<eps) .and. (abs(g(2,ig)) <eps) ) THEN
        !
        !    a vector of the 1D grid has been found
        !
        ig1d = ig1d+1
        IF (ig1d>dfftp%nr3) CALL errore ('ggen1d', 'too many G', 1)
        g1d (3, ig1d) = g (3, ig)
        gg1d (ig1d) = gg (ig)
        ig1dto3d (ig1d) = ig
        nl1d (ig1d) = nint (g (3, ig) * at (3, 3) ) + 1
        IF (nl1d (ig1d) <1) nl1d (ig1d) = nl1d (ig1d) + dfftp%nr3
     ENDIF
  ENDDO

  ngm1d = ig1d
  DO ig1d = 1, ngm1d
     igtongl1d (ig1d) = igtongl (ig1dto3d (ig1d) )
  ENDDO
  RETURN
END SUBROUTINE ggen1d
