/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

! Copyright (C) 2006-2008 Dmitry Korotin dmitry@korotin.name
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
SUBROUTINE wannier_u_matrix(U,hJ)

  USE io_global, ONLY: stdout, ionode, ionode_id
  USE io_files
  USE kinds, ONLY: DP
  USE wannier_new, ONLY: nwan, pp, wannier_occ, wannier_energy,wan_in

  IMPLICIT NONE
  INTEGER i,j, k,c, iwan, l
  real(DP) :: U, hJ, u2(10,10)

  INTEGER :: atoms(10)
  real(DP) :: rotm(10,10), unew(10,10), tmp

  WRITE(stdout,'(5x,a34)') 'Generation of interaction matrix U'
  WRITE(stdout,'(5x,a29)') '(works only for nspin=1 case)'
  WRITE(stdout,*)
  u2 = 0.d0
  CALL mk_u(2,5,U,hJ,u2)

  !rotation from TB-LMTO basis to our new

  rotm = 0.d0
  c = 0
  DO iwan=1, nwan
     DO j=1,wan_in(iwan,1)%ning
        IF(wan_in(iwan,1)%ing(j)%l==2) THEN
           c = c+1
           SELECT CASE(wan_in(iwan,1)%ing(j)%m)
           CASE(1)
              rotm(c,3) = wan_in(iwan,1)%ing(j)%c
           CASE(2)
              rotm(c,4) = wan_in(iwan,1)%ing(j)%c
           CASE(3)
              rotm(c,2) = wan_in(iwan,1)%ing(j)%c
           CASE(4)
              rotm(c,5) = wan_in(iwan,1)%ing(j)%c
           CASE(5)
              rotm(c,1) = wan_in(iwan,1)%ing(j)%c
           END SELECT
        ENDIF
     ENDDO
  ENDDO

  IF(c>5) CALL errore('Too many interactiong atoms - cant construct U matrix',c)

  DO i=1,5
     DO j=1,5
        rotm(i+5,j+5) = rotm(i,j)
     ENDDO
  ENDDO

  DO i = 1,10
     DO j = 1, 10
        tmp = 0.d0
        DO k=1,10
           DO l=1,10
              tmp=tmp+rotm(i,k)*u2(k,l)*rotm(j,l)
           ENDDO
        ENDDO
        unew(i,j)=tmp
     ENDDO
  ENDDO

!output
  DO i=1,c
     WRITE(stdout,'(5x,10f5.2)') (unew(i,j),j=1,c), (unew(i,j+5),j=1,c)
  ENDDO
  DO i=6,5+c
     WRITE(stdout,'(5x,10f5.2)') (unew(i,j),j=1,c), (unew(i,j+5),j=1,c)
  ENDDO
  WRITE(stdout,*)

  OPEN(70,file='umatrix',status='unknown',form='formatted')
  DO i=1,c
     WRITE(70,'(10f5.2)') (unew(i,j),j=1,c), (unew(i,j+5),j=1,c)
  ENDDO
  DO i=6,5+c
     WRITE(70,'(10f5.2)') (unew(i,j),j=1,c), (unew(i,j+5),j=1,c)
  ENDDO
  WRITE(70,*)
  CLOSE(70)

END SUBROUTINE wannier_u_matrix
