/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2006 Malgorzata Wierbowska and Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
MODULE a2F
  !
  ! This module contains a routine saving variables needed for the
  ! electron-phonon calculation (new algorithm implemeted by MW) 
  !
  USE kinds,      ONLY : DP
  !
  LOGICAL :: la2F = .FALSE.
  !
  PRIVATE
  PUBLIC :: la2F, a2Fsave
  CONTAINS
    !
    SUBROUTINE a2Fsave
  
      USE kinds,        ONLY : DP
      USE klist,        ONLY : nks, nkstot, xk, wk
      USE ions_base,    ONLY : nat
      USE wvfct,        ONLY : et, nbnd
      USE start_k,      ONLY : nk1, nk2, nk3 
      USE symm_base,    ONLY : s, nsym, irt
      USE io_global,    ONLY : ionode
      USE io_files,     ONLY : seqopn
      implicit none
      !
      INTEGER :: iuna2Fsave  = 40, i, j, ik, ns, na
      logical  ::  exst
      !
      ! parallel case: only first node writes
      IF ( ionode ) THEN
         !
         CALL seqopn( iuna2Fsave, 'a2Fsave', 'FORMATTED', exst )
         !===========================================
         !
         WRITE( iuna2Fsave, * ) nbnd, nkstot
         WRITE( iuna2Fsave, * ) et
         WRITE( iuna2Fsave, * ) ((xk(i,ik), i=1,3), ik=1,nkstot)
         WRITE( iuna2Fsave, * ) wk(1:nkstot)
         WRITE( iuna2Fsave, * ) nk1, nk2, nk3
         !
         WRITE( iuna2Fsave, * ) nsym
         do ns=1,nsym
            WRITE( iuna2Fsave, * )  ((s(i,j,ns),j=1,3),i=1,3) 
         enddo
         WRITE( iuna2Fsave, * )  ((irt(ns,na),ns=1,nsym),na=1,nat)
         !
         CLOSE( UNIT = iuna2Fsave, STATUS = 'KEEP' )
         !  
      END IF
      !
      RETURN
    END SUBROUTINE a2Fsave
  END MODULE a2F
