/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2009-2010 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!---------------------------------------------------------------------------
SUBROUTINE compute_deff(deff, et)
!
!  This routine computes the effective value of the D-eS coefficients
!  which appear often in many expressions in the US or PAW case. 
!  This routine is for the collinear case.
!
USE kinds, ONLY : DP
USE ions_base, ONLY : nsp, nat, ityp
USE uspp, ONLY : deeq, qq, okvan
USE uspp_param, ONLY : nhm
USE lsda_mod, ONLY : current_spin
IMPLICIT NONE

INTEGER :: nt, na, is
REAL(DP), INTENT(OUT) :: deff(nhm, nhm, nat) 
REAL(DP), INTENT(IN) :: et

deff(:,:,:) = deeq(:,:,:,current_spin) 
IF (okvan) THEN
   DO nt = 1, nsp
      DO na = 1, nat
         IF ( ityp(na) == nt ) THEN
            deff(:,:,na) = deff(:,:,na) - et*qq(:,:,nt)
         END IF
      END DO
   END DO
ENDIF
RETURN
END SUBROUTINE compute_deff
!
SUBROUTINE compute_deff_nc(deff, et)
!
!  This routine computes the effective value of the D-eS coefficients
!  which appears often in many expressions. This routine is for the
!  noncollinear case.
!
USE kinds, ONLY : DP
USE ions_base, ONLY : nsp, nat, ityp
USE spin_orb,  ONLY : lspinorb
USE noncollin_module, ONLY : noncolin, npol
USE uspp, ONLY : deeq_nc, qq, qq_so, okvan
USE uspp_param, ONLY : nhm
USE lsda_mod, ONLY : nspin
IMPLICIT NONE

INTEGER :: nt, na, is, js, ijs
COMPLEX(DP), INTENT(OUT) :: deff(nhm, nhm, nat, nspin) 
REAL(DP), INTENT(IN) :: et

deff=deeq_nc
IF (okvan) THEN
   DO nt = 1, nsp
      DO na = 1, nat
         IF ( ityp(na) == nt ) THEN
            IF (lspinorb) THEN
               deff(:,:,na,:) = deff(:,:,na,:) - et * qq_so(:,:,:,nt)
            ELSE
               ijs=0
               DO is=1,npol
                  DO js=1,npol
                     ijs=ijs+1
                     IF (is==js) deff(:,:,na,ijs)=deff(:,:,na,ijs)-et*qq(:,:,nt)
                  END DO
               END DO
            END IF
         END IF
      END DO
   END DO
ENDIF

RETURN
END SUBROUTINE compute_deff_nc

