/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2005-2012 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
   SUBROUTINE compute_rho(rho,rhoout,segni,nrxx)
!
!   This subroutine diagonalizes the spin density matrix and gives as output
!   the spin up and spin down compotents of the charge
!   If lsign is true up and dw are with respect to the fixed quantization
!   axis ux, otherwise rho+|m| is always rhoup and rho-|m| is always
!   rhodw
!
   USE kinds, ONLY : dp
   USE noncollin_module, ONLY : lsign, ux
   IMPLICIT NONE
   INTEGER :: nrxx  ! input: the dimension of the mesh
         
   REAL(DP), INTENT(IN) ::  rho(nrxx,4)
   REAL(DP), INTENT(OUT) :: rhoout(nrxx,2)
   REAL(DP), INTENT(OUT) :: segni(nrxx)
                 ! input: the four components of the charge 
                 ! output: the spin up and spin down charge
                 ! output: the orientation when needed
   REAL(DP) :: amag
   INTEGER :: ir           ! counter on mesh points

   segni=1.0_DP
   IF (lsign) THEN
      DO ir=1,nrxx
         segni(ir)=SIGN(1.0_DP,rho(ir,2)*ux(1)+rho(ir,3)*ux(2)+rho(ir,4)*ux(3))
      ENDDO
   ENDIF

   DO ir=1,nrxx
      amag=SQRT(rho(ir,2)**2+rho(ir,3)**2+rho(ir,4)**2)
      rhoout(ir,1)=0.5d0*(rho(ir,1)+segni(ir)*amag)
      rhoout(ir,2)=0.5d0*(rho(ir,1)-segni(ir)*amag)
   ENDDO

   RETURN
END SUBROUTINE compute_rho
