/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
subroutine coset (nrot, table, sym, nsym, irg)
  !-----------------------------------------------------------------------
  !
  !  Divides the elements of a given group into left cosets of one
  !  of its subgroups.
  !  The input is the array sym which is true only for the
  !  operations of the subgroup, the output is nsym, and the array irg,
  !  which contains as its first elements the indices of the subgroup,
  !  and then its right cosets.
  !
  !  revised layout 1 may 1995 by A. Dal Corso
  !
  USE kinds
  implicit none
  !
  !    first the dummy variables
  !
  integer :: nrot, table (48, 48), nsym, irg (48)
  ! input: order of the group
  ! input: multiplication table of the group
  ! output: order of the subgroup
  ! output: gives the correspondence of symme
  ! operations forming a n-th coset
  ! input: flag indicating if an operations
  logical :: sym (48)
  ! belongs to the subgroup
  !
  ! here the local variables
  !
  logical :: done (48)
  ! if true the operation has been already ch

  integer :: irot, ncos, isym, nc, nelm
  ! counter on rotations
  ! number of cosets (=nrot/nsym)
  ! counter on symmetries
  ! counter on cosets
  ! counter on the number of elements
  !
  !    here we count the elements of the subgroup and set the first part o
  !    irg which contain the subgroup
  !
  nsym = 0
  do irot = 1, nrot
     done (irot) = sym (irot)
     if (sym (irot) ) then
        nsym = nsym + 1
        irg (nsym) = irot
     endif
  enddo
  !
  !     we check that the order of the subgroup is a divisor of the order
  !     total group. ncos is the number of cosets
  !
  IF ( nsym == 0 ) CALL errore( 'coset', 'nsym == 0', 1 ) 
  !
  ncos = nrot / nsym
  if (ncos * nsym.ne.nrot) call errore ('coset', &
  'The order'//' of the group is not a multiple of that of the subgroup', 1)
  !
  !     here we set the other elements of irg, by using the multiplication
  !
  nelm = nsym
  do nc = 2, ncos
     do irot = 1, nrot
        if (.not.done (irot) ) then
           do isym = 1, nsym
              nelm = nelm + 1
              irg (nelm) = table (irot, irg (isym) )
              done (irg (nelm) ) = .true.
           enddo
        endif
     enddo

  enddo
  return
end subroutine coset
