/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2012 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
SUBROUTINE divide (comm, ntodiv, startn, lastn)
  !-----------------------------------------------------------------------
  ! Divide ntodiv poins across processors belonging to communicator comm 
  ! Each processor gets points from startn to lastn
  !
  !
  USE mp, ONLY : mp_size, mp_rank
  IMPLICIT NONE
  !
  INTEGER, INTENT(in) :: comm
  INTEGER, INTENT(in) :: ntodiv
  INTEGER, INTENT(out):: startn, lastn
  !
  INTEGER :: me_comm, nproc_comm
  !
  INTEGER :: nb, resto, idx, ip
  ! number of bands per processor
  ! one additional band if me_pool+1 <= resto
  ! counter on bands
  ! counter on processors
  !
  nproc_comm = mp_size(comm)
  me_comm = mp_rank(comm)
  !
  nb = ntodiv / nproc_comm
  resto = ntodiv - nb * nproc_comm
  idx = 0
  DO ip = 1, nproc_comm
     IF (ip <= resto) THEN
        IF (me_comm+1 == ip) THEN
           startn = idx + 1
           lastn = startn + nb
        ENDIF
        idx = idx + nb + 1
     ELSE
        IF (me_comm+1 == ip) THEN
           startn = idx + 1
           lastn = startn + nb - 1
        ENDIF
        idx = idx + nb
     ENDIF
  ENDDO
  RETURN

END SUBROUTINE divide

!
!-----------------------------------------------------------------------
SUBROUTINE divide2 (comm1, comm2, ntodiv, startn, lastn)
  !-----------------------------------------------------------------------
  ! Divide ntodiv points across processors belonging to two communicators 
  ! comm1 and comm2. The final quantity must be collected among the two
  ! Each processor gets points from startn to lastn
  !
  !
  USE mp, ONLY : mp_size, mp_rank
  IMPLICIT NONE
  !
  INTEGER, INTENT(in) :: comm1
  INTEGER, INTENT(in) :: comm2
  INTEGER, INTENT(in) :: ntodiv
  INTEGER, INTENT(out):: startn, lastn

  INTEGER :: ntodiv1, start_n1, end_n1, start_n2, end_n2
  !
  CALL divide( comm1, ntodiv, start_n1, end_n1 )
  ntodiv1=end_n1-start_n1+1
  CALL divide( comm2, ntodiv1, start_n2, end_n2 )
  startn=start_n1+start_n2-1
  lastn=start_n1+end_n2-1

  RETURN

END SUBROUTINE divide2

