/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2005 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!---------------------------------------------------------------------------
      subroutine get_locals(rholoc,magloc, rho)
!---------------------------------------------------------------------------
!
! Here local integrations are carried out around atoms.
! The points and weights for these integrations are determined in the
! subroutine make_pointlists, the result may be printed in the
! subroutine report_mag. If constraints are present, the results of this
! calculation are used in v_of_rho for determining the penalty functional.
!
      USE kinds,      ONLY : DP
      USE ions_base,  ONLY : nat
      USE cell_base,  ONLY : omega
      USE lsda_mod,   ONLY : nspin
      USE mp_bands,   ONLY : intra_bgrp_comm
      USE mp,         ONLY : mp_sum
      USE fft_base,   ONLY : dfftp
      USE noncollin_module, ONLY : pointlist, factlist, noncolin

      implicit none
!
! I/O variables
!
      real(DP) ::   &
          rholoc(nat),   &     ! integrated charge arount the atoms
          magloc(nspin-1,nat)  ! integrated magnetic moment around the atom
      real(DP) :: rho (dfftp%nnr, nspin)
!
! local variables
!
      integer i,ipol
      real(DP) :: fact
      real(DP), allocatable :: auxrholoc(:,:)
 
      allocate (auxrholoc(0:nat,nspin))
      auxrholoc(:,:) = 0.d0
      do i=1,dfftp%nnr
         auxrholoc(pointlist(i),1:nspin) = auxrholoc(pointlist(i),1:nspin) + &
                                           rho(i,1:nspin) * factlist(i)
      end do
      !
      call mp_sum( auxrholoc( 0:nat, 1:nspin), intra_bgrp_comm )
      !     
      fact =  omega/(dfftp%nr1*dfftp%nr2*dfftp%nr3)
      if (nspin.eq.2) then
         rholoc(1:nat)   = (auxrholoc(1:nat,1)+auxrholoc(1:nat,2)) * fact
         magloc(1,1:nat) = (auxrholoc(1:nat,1)-auxrholoc(1:nat,2)) * fact
      else
         rholoc(1:nat) = auxrholoc(1:nat,1) * fact
         if (noncolin) then
            do ipol=1,3
               magloc(ipol,1:nat) = auxrholoc(1:nat,ipol+1) * fact
            end do
         end if
      endif
!
      deallocate (auxrholoc)

      end subroutine get_locals
