/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001-2010 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
SUBROUTINE multable (nsym, s, table)
  !-----------------------------------------------------------------------
  !
  !  Checks that {S} is a group and calculates multiplication table
  !
  IMPLICIT NONE
  !
  INTEGER, INTENT(IN) :: nsym, s(3,3,nsym)
  ! nsym = number of symmetry operations
  ! s    = rotation matrix (in crystal axis, represented by integers)
  INTEGER, INTENT(OUT) :: table (48, 48)
  ! multiplication table:  S(n)*S(m) = S (table(n,m) )
  !
  INTEGER :: isym, jsym, ksym, ss (3, 3)
  LOGICAL :: found, smn
  !
  DO isym = 1, nsym
     DO jsym = 1, nsym
        ! 
        ss = MATMUL (s(:,:,jsym),s(:,:,isym))
        !
        !     here we check that the input matrices really form a group
        !     and we set the multiplication table
        !
        found = .false.
        DO ksym = 1, nsym
           smn =  ALL ( s(:,:,ksym) == ss(:,:) )
           IF (smn) THEN
              IF (found) CALL errore ('multable', 'Not a group', 1)
              found = .true.
              table (jsym, isym) = ksym
           END IF
        END DO
        IF ( .NOT.found) CALL errore ('multable', ' Not a group', 2)
     END DO
  END DO
  RETURN
  !
END SUBROUTINE multable
