/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
subroutine n_plane_waves (ecutwfc, tpiba2, nks, xk, g, ngm, npwx, ngk)
  !-----------------------------------------------------------------------
  !
  ! Find number of plane waves for each k-point
  !
  USE kinds, only: DP
  USE mp,       ONLY : mp_max
  USE mp_pools, ONLY : inter_pool_comm
  implicit none
  !
  integer, intent(in) :: nks, ngm
  real(DP),intent(in) :: ecutwfc, tpiba2, xk (3, nks), g (3, ngm)
  !
  integer, intent(out) :: npwx, ngk (nks)
  !
  integer :: nk, ng
  real(DP) :: q2
  !
  npwx = 0
  do nk = 1, nks
     ngk (nk) = 0
     do ng = 1, ngm
        q2 = (xk (1, nk) + g (1, ng) ) **2 + (xk (2, nk) + g (2, ng) ) ** &
             2 + (xk (3, nk) + g (3, ng) ) **2
        if (q2 <= ecutwfc / tpiba2) then
           !
           ! here if |k+G|^2 <= Ecut increase the number of G inside the sphere
           !
           ngk (nk) = ngk (nk) + 1
        else
           if (sqrt (g (1, ng) **2 + g (2, ng) **2 + g (3, ng) **2) &
                .gt.sqrt (xk (1, nk) **2 + xk (2, nk) **2 + xk (3, nk) **2) &
                + sqrt (ecutwfc / tpiba2) ) goto 100
           !
           ! if |G| > |k| + sqrt(Ecut)  stop search
           !
        endif
     enddo
100  npwx = max (npwx, ngk (nk) )
  enddo
  if (npwx <= 0) call errore ('n_plane_waves', &
                'No plane waves found: running on too many processors?', 1)
  !
  ! when using pools, set npwx to the maximum value across pools
  ! (you may run into trouble at restart otherwise)
  !
  CALL mp_max ( npwx, inter_pool_comm )
  !
  return
end subroutine n_plane_waves
