/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!-----------------------------------------------------------------------
subroutine ns_adj
!-----------------------------------------------------------------------
! This routine tries to suggest to the code the right atomic orbital to 
! localize the charge on.
!
   USE kinds,     ONLY : DP
   USE ions_base, ONLY : nat, ntyp => nsp, ityp
   USE ldaU,      ONLY : Hubbard_lmax, Hubbard_l, Hubbard_U, starting_ns
   USE scf,       ONLY : rho
   USE lsda_mod,  ONLY : nspin
   USE noncollin_module, ONLY : noncolin, npol
   USE io_global, ONLY : stdout
 
   implicit none
   !
   integer, parameter:: ldmx=7
   integer :: na, nt, is, m1, m2, ldim, i, j, l 
   real(DP) :: lambda(npol*ldmx) 
   complex(DP) :: vet(npol*ldmx,npol*ldmx), f(npol*ldmx,npol*ldmx), temp
 
   if (ALL(starting_ns == -1.d0)) return
   write (stdout,*) "Modify starting ns matrices according to input values "
 
   if (2*Hubbard_lmax+1>ldmx) call errore('ns_adj',' ldmx too small',ldmx) 

   do na = 1, nat
      nt = ityp(na)
      if (Hubbard_U(nt).ne.0.d0) then
       ldim = 2 * Hubbard_l(nt) + 1 

       if (noncolin) then

         do m1 = 1, ldim
            do m2 = 1, ldim
              f(m1, m2)           = rho%ns_nc(m1, m2, 1, na)
              f(m1, ldim+m2)      = rho%ns_nc(m1, m2, 2, na)
              f(ldim+m1, m2)      = rho%ns_nc(m1, m2, 3, na)
              f(ldim+m1, ldim+m2) = rho%ns_nc(m1, m2, 4, na)
            end do
         end do
         call cdiagh( npol*ldim, f, npol*ldmx, lambda, vet)
         j = 0 
         do is = 1, npol
           do i = 1, ldim
             j = j + 1
             if (starting_ns(i,is,nt) >= 0.d0) lambda(j) = starting_ns(i,is,nt)
           enddo
         enddo
         do m1 = 1, npol*ldim
            do m2 = m1, npol*ldim
               temp = 0.d0
               do i = 1, npol*ldim
                  temp = temp + vet(m1,i)*lambda(i)*CONJG(vet(m2,i))     
               end do
               f(m1,m2) =  temp
               f(m2,m1) =  CONJG(temp) 
            end do
         end do
         do m1 = 1, ldim
            do m2 = 1, ldim
               rho%ns_nc(m1, m2, 1, na) = f(m1, m2) 
               rho%ns_nc(m1, m2, 2, na) = f(m1, ldim+m2) 
               rho%ns_nc(m1, m2, 3, na) = f(ldim+m1, m2) 
               rho%ns_nc(m1, m2, 4, na) = f(ldim+m1, ldim+m2) 
            end do
         end do

       else

         do is = 1, nspin
            do m1 = 1, ldim
               do m2 = 1, ldim 
                  f(m1,m2) = rho%ns(m1,m2,is,na)
               enddo
            enddo
            call cdiagh(ldim, f, ldmx, lambda, vet)
            do i = 1, ldim
               if (starting_ns(i,is,nt) >= 0.d0) lambda(i) = starting_ns(i,is,nt)
            enddo
            do m1 = 1, ldim
               do m2 = m1, ldim
                  temp = 0.d0
                  do i = 1, ldim
                     temp = temp + CONJG(vet(m1,i))*lambda(i)*vet(m2,i)
                  enddo
                  rho%ns(m1,m2,is,na) =  DBLE(temp)
                  rho%ns(m2,m1,is,na) = rho%ns(m1,m2,is,na)
               enddo
            enddo
         enddo

       endif 

      endif
   enddo ! on na

   if (noncolin) then
     CALL write_ns_nc
   else
     CALL write_ns
   endif

   return
end subroutine ns_adj
