/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2001-2008 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
SUBROUTINE offset_atom_wfc( Hubbard_only, offset, counter )
  !----------------------------------------------------------------------------
  !
  ! For each Hubbard atom, compute the index of the projector in the list of
  ! atomic wavefunctions. IMPORTANT NOTICE: if there is more than one state
  ! with the chosen value of Hubbard_l, the one selected for U calculation is
  ! the last state with the given l, but with strictly positive occupation, 
  ! If Hubbard_only=.true., find offsets with respect to the array of atomic
  ! wavefunctions including only those with a Hubbard U term (wfcU in ldaU.f90)
  !
  USE uspp_param,       ONLY : upf
  USE noncollin_module, ONLY : noncolin
  USE ions_base,        ONLY : nat, ityp
  USE ldaU,             ONLY : Hubbard_l, Hubbard_U, Hubbard_alpha
  IMPLICIT NONE
  !
  LOGICAL, INTENT(IN)  :: Hubbard_only
  INTEGER, INTENT(OUT) :: offset(nat), counter
  !
  INTEGER  :: na, nt, n, l
  LOGICAL  :: hubbard_wfc
  !
  !
  counter = 0
  offset(:) = -99
  !
  DO na = 1, nat
     !
     nt = ityp(na)
     !
     DO n = 1, upf(nt)%nwfc
        !
        IF ( upf(nt)%oc(n) >= 0.D0 ) THEN
           !
           l = upf(nt)%lchi(n)
           hubbard_wfc = ( upf(nt)%oc(n)>0.D0 .AND. l == Hubbard_l(nt) )
           !
           IF ( noncolin ) THEN
              !
              IF ( upf(nt)%has_so ) THEN
                 !
                 ! offset to be set at the first occurrence of required l
                 IF (hubbard_wfc .AND. offset(na).eq.-99 ) offset(na) = counter
                 !
                 IF (hubbard_wfc .OR. .NOT. hubbard_only) THEN
                    ! j = l-1/2, degeneracy 2l
                    counter = counter + 2*l
                    ! j = l+1/2, degeneracy 2*l+2
                    IF (ABS( upf(nt)%jchi(n)-l-0.5D0 ) < 1.D-6) &
                      counter = counter + 2
                 END IF
                 !
                 IF (hubbard_wfc .AND. hubbard_only) THEN
                    counter = counter + 2*l + 2
                 END IF
                 !
              ELSE
                 !
                 IF (hubbard_wfc) offset(na) = counter
                 !
                 IF (hubbard_wfc .OR. .NOT. hubbard_only) THEN
                    counter = counter + 2*( 2*l + 1 )
                 END IF
                 !
              END IF
              !
           ELSE
              !
              IF (hubbard_wfc) offset(na) = counter
              !
              IF (hubbard_wfc .OR. .NOT. hubbard_only) THEN
                 counter = counter + 2*l + 1
              END IF
              !
           END IF
        END IF
     END DO
     
     IF ( (Hubbard_U(nt).NE.0.D0 .OR. Hubbard_alpha(nt).NE.0.D0 ) .AND. &
         offset(na) < 0 ) CALL errore('offset_atom_wfc', 'wrong offset', na)

  END DO
  !
  RETURN
  !
END SUBROUTINE offset_atom_wfc
!
