/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2011 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
! This routine is inspired by the former routine pw2casino of
! Norbert Nemec
! (C) 2010 by Norbert Nemec <Norbert@Nemec-online.de> 
!----------------------------------------------------------------------------
SUBROUTINE pw2casino()
  !----------------------------------------------------------------------------
  !
  USE kinds,         ONLY : DP
  !
  USE mp_images,     ONLY : nimage
  USE mp_bands,      ONLY : nbgrp
  USE mp_pools,      ONLY : npool
  !
  USE control_flags, ONLY : istep, nstep
  !
  USE io_files, ONLY : tmp_dir
  !
  USE plugin_flags, ONLY : use_pw2casino
  !
  IMPLICIT NONE
  !
  CHARACTER(len=4) :: postfix
  !
  CHARACTER(len=6), EXTERNAL :: int_to_char
  !
  INTEGER, EXTERNAL :: find_free_unit
  !
  INTEGER :: tmp_unit
  !
  INTEGER  :: ios
  LOGICAL  :: casino_gather = .true.
  LOGICAL  :: blip_convert = .true.
  LOGICAL  :: blip_binary = .true.  
  LOGICAL  :: blip_single_prec = .false.
  REAL(dp) :: blip_multiplicity = 1.d0
  INTEGER  :: n_points_for_test = 0
  !
  NAMELIST / inputpp / &
   blip_convert, &
   blip_multiplicity, &
   blip_binary, &
   blip_single_prec, &
   n_points_for_test
  !
  !
  IF ( use_pw2casino ) THEN
    !
    IF ( npool > 1 .or. nimage > 1 .or. nbgrp > 1 ) THEN
      CALL errore('pw2casino', 'pool/band/image parallelization not (yet) implemented',1)
    ENDIF
    !
    tmp_unit = find_free_unit()
    OPEN(unit=tmp_unit,file = trim(tmp_dir)//'/'//'pw2casino.dat',status='old',err=20)
    READ(tmp_unit,inputpp,iostat=ios)
    CLOSE(tmp_unit)

20 CONTINUE
 
   IF ( .not. blip_convert ) blip_binary = .false.
 
    IF ( nstep == 1 ) THEN
      write(postfix,*) ''
      CALL write_casino_wfn( &
               casino_gather, & ! gather
               blip_convert,  & ! blip
               blip_multiplicity,   & ! multiplicity
               blip_binary,  & ! binwrite
               blip_single_prec,  & ! single_precision_blips
               n_points_for_test,       & ! n_points_for_test
               postfix)   ! postfix

    ELSE
!      write(postfix,'(i4.4)') istep
      postfix=trim(int_to_char(istep))
      !
      CALL write_casino_wfn( &
               casino_gather, & ! gather
               blip_convert,  & ! blip
               blip_multiplicity,   & ! multiplicity
               blip_binary,  & ! binwrite
               blip_single_prec,  & ! single_precision_blips
               n_points_for_test,       & ! n_points_for_test
               '.'//postfix)   ! postfix
    ENDIF
  ENDIF
  !
  !
END SUBROUTINE pw2casino
