/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2004 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
function sph_ind(l,j,m,spin)
! This function calculates the m index of the spherical harmonic
! in a spinor with orbital angular momentum l, total angular 
! momentum j, projection along z of the total angular momentum m+-1/2. 
! Spin selects the up (spin=1) or down (spin=2) coefficient.
!
use kinds
implicit none

integer :: sph_ind
integer :: l, &            ! orbital angular momentum
           m, &            ! projection of the total angular momentum+-1/2
           spin            ! 1 or 2 select the component

real(DP) :: j         ! total angular momentum

if (spin.ne.1.and.spin.ne.2) call errore('sph_ind','spin direction unknown',1)
if (m.lt.-l-1.or.m.gt.l) call errore('sph_ind','m not allowed',1)

if (abs(j-l-0.5d0).lt.1.d-8) then
   if (spin.eq.1) sph_ind= m
   if (spin.eq.2) sph_ind= m+1
elseif (abs(j-l+0.5d0).lt.1.d-8) then
   if (m.lt.-l+1) then
      sph_ind=0
   else
      if (spin.eq.1) sph_ind= m-1
      if (spin.eq.2) sph_ind= m
   endif
else
   write(6,*) l, j
   call errore('sph_ind','l and j not compatible',1)
endif
if (sph_ind.lt.-l.or.sph_ind.gt.l) sph_ind=0

return
end function sph_ind
