/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2006 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!----------------------------------------------------------------------------
SUBROUTINE transform_becsum_nc(becsum_nc,becsum,na)
!----------------------------------------------------------------------------
!
! This routine multiply becsum_nc by the identity and the Pauli
! matrices and saves it in becsum for the calculation of 
! augmentation charge and magnetization.
!
USE kinds,                ONLY : DP
USE ions_base,            ONLY : nat, ntyp => nsp, ityp
USE uspp_param,           ONLY : nh, nhm
USE lsda_mod,             ONLY : nspin
USE noncollin_module,     ONLY : npol, nspin_mag
USE spin_orb,             ONLY : domag
!
IMPLICIT NONE

COMPLEX(DP) :: becsum_nc(nhm*(nhm+1)/2,nat,npol,npol)
REAL(DP) :: becsum(nhm*(nhm+1)/2,nat,nspin_mag)
INTEGER :: na
!
! ... local variables
!
INTEGER :: ih, jh, ijh, np

np=ityp(na)
ijh=1
DO ih = 1, nh(np)
   becsum(ijh,na,1)= becsum(ijh,na,1)+  &
               becsum_nc(ijh,na,1,1)+becsum_nc(ijh,na,2,2)
   IF (domag) THEN
      becsum(ijh,na,2)= becsum(ijh,na,2)+  &
               becsum_nc(ijh,na,1,2)+becsum_nc(ijh,na,2,1)
      becsum(ijh,na,3)= becsum(ijh,na,3)+(0.d0,-1.d0)*  &
              (becsum_nc(ijh,na,1,2)-becsum_nc(ijh,na,2,1))
      becsum(ijh,na,4)= becsum(ijh,na,4)+  &
               becsum_nc(ijh,na,1,1)-becsum_nc(ijh,na,2,2)
   END IF
   ijh=ijh+1
   DO jh = ih+1, nh(np)
      becsum(ijh,na,1)= becsum(ijh,na,1) +     &
                 (becsum_nc(ijh,na,1,1)+becsum_nc(ijh,na,2,2))  &
                + CONJG(becsum_nc(ijh,na,1,1)+becsum_nc(ijh,na,2,2))
      IF (domag) THEN
         becsum(ijh,na,2)= becsum(ijh,na,2) +     &
                  becsum_nc(ijh,na,1,2)+becsum_nc(ijh,na,2,1)   &
                + CONJG(becsum_nc(ijh,na,2,1)+becsum_nc(ijh,na,1,2))
         becsum(ijh,na,3)= becsum(ijh,na,3) +(0.d0,-1.d0)*     &
                 (becsum_nc(ijh,na,1,2)-becsum_nc(ijh,na,2,1)   &
                + CONJG(becsum_nc(ijh,na,2,1)-becsum_nc(ijh,na,1,2)) )
         becsum(ijh,na,4)= becsum(ijh,na,4) +     &
                 (becsum_nc(ijh,na,1,1)-becsum_nc(ijh,na,2,2))  &
                + CONJG(becsum_nc(ijh,na,1,1)-becsum_nc(ijh,na,2,2))
      END IF
      ijh=ijh+1
   END DO
END DO

RETURN
END SUBROUTINE transform_becsum_nc
