/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */


! Copyright (C) 2012 Quantum-Espresso group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!----------------------------------------------------------------------
SUBROUTINE transform_qq_so(qq,qq_so)
  !----------------------------------------------------------------------
  !
  !
  USE kinds,        ONLY : DP
  USE ions_base,    ONLY : ntyp => nsp
  USE uspp_param,   ONLY : upf, nhm, nh
  USE spin_orb,     ONLY : lspinorb, fcoef
  !
  implicit none
  !
  !     here a few local variables
  !
  integer :: nt, ih, jh, kh, lh, ijs, is1, is2, is
  complex(DP) :: qq(nhm,nhm,ntyp), qq_so(nhm,nhm,4,ntyp)

  qq_so=(0.0_DP, 0.0_DP)
  DO nt = 1, ntyp
    IF ( upf(nt)%tvanp ) THEN
      IF (upf(nt)%has_so) THEN
        DO ih=1,nh(nt)
          DO jh=1,nh(nt)
            DO kh=1,nh(nt)
              DO lh=1,nh(nt)
                ijs=0
                DO is1=1,2
                  DO is2=1,2
                    ijs=ijs+1
                    DO is=1,2
                      qq_so(kh,lh,ijs,nt) = qq_so(kh,lh,ijs,nt)       &
                          + qq(ih,jh,nt)*fcoef(kh,ih,is1,is,nt)&
                                               *fcoef(jh,lh,is,is2,nt)
                    ENDDO
                  ENDDO
                ENDDO
              ENDDO
            ENDDO
          ENDDO
        ENDDO
      ELSE
        DO ih = 1, nh (nt)
          DO jh = ih, nh (nt)
             IF (lspinorb) THEN
                 qq_so (ih, jh, 1, nt) = qq (ih, jh, nt) 
                 qq_so (jh, ih, 1, nt) = qq_so (ih, jh, 1, nt)
                 qq_so (ih, jh, 4, nt) = qq_so (ih, jh, 1, nt)
                 qq_so (jh, ih, 4, nt) = qq_so (ih, jh, 4, nt)
             ENDIF
          ENDDO
        ENDDO
      ENDIF
    ENDIF
  ENDDO

  RETURN
END SUBROUTINE transform_qq_so
