/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

! Copyright (C) 2008 Dmitry Korotin dmitry@korotin.name
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!


!----------------------------------------------------------------------
subroutine wannier_clean()
  !----------------------------------------------------------------------
  !    
  ! ... This routine deallocates all dynamically allocated arrays for wannier calc and closes all open files
  !
  USE wannier_new, only: wan_in, wan_pot, wannier_energy, wannier_occ, pp, coef
  USE io_files
  USE buffers
  USE basis,      ONLY : swfcatom
  USE ldaU,       ONLY : lda_plus_u
  USE fixed_occ, ONLY : one_atom_occupations
  
  IMPLICIT NONE
  LOGICAL :: opnd
  
  if(allocated(wan_in)) deallocate(wan_in)
  if(allocated(wan_pot)) deallocate(wan_pot)
  if(allocated(wannier_energy)) deallocate(wannier_energy)
  if(allocated(wannier_occ)) deallocate(wannier_occ)
  if(allocated(pp)) deallocate(pp)
  if(allocated(coef)) deallocate(coef)

  CALL close_buffer( iunwpp, 'keep' )
  CALL close_buffer( iunwf, 'keep' )
  
  IF ( .NOT. ( lda_plus_u .OR. one_atom_occupations ) ) THEN
     INQUIRE( UNIT = iunsat, OPENED = opnd )  
     IF ( opnd ) CALL close_buffer( iunsat, 'delete' )
  END IF
  INQUIRE( UNIT = iunigk, OPENED = opnd )  
  IF ( opnd ) CALL close_buffer( iunigk, 'delete' )
  
  IF(ALLOCATED(swfcatom)) DEALLOCATE(swfcatom)

  return
  !
end subroutine wannier_clean

