/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

! Copyright (C) 2008 Dmitry Korotin dmitry@korotin.name
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!

!----------------------------------------------------------------------
SUBROUTINE wannier_init(hwwa)
  !----------------------------------------------------------------------
  !    
  ! ... This routine ALLOCATEs all dynamically ALLOCATEd arrays for wannier calc
  !
  USE wannier_new 
  USE wvfct, only : nbnd, npwx
  USE input_parameters, only: constrain_pot, wan_data
  USE lsda_mod, only: nspin
  USE ions_base, only : nat
  USE basis, only : natomwfc, swfcatom
  USE constants, only: rytoev
  USE klist, only: nks
  USE io_files
  USE buffers
  USE noncollin_module, ONLY : npol

  IMPLICIT NONE 
  
  LOGICAL,INTENT(IN) :: hwwa ! have we Wannier already?
  LOGICAL :: exst = .FALSE.,opnd
  INTEGER :: i, io_level

  ALLOCATE(pp(nwan,nbnd))
  ALLOCATE(wan_in(nwan,nspin))
  ALLOCATE(wannier_energy(nwan,nspin))
  ALLOCATE(wannier_occ(nwan,nwan,nspin))
  ALLOCATE(coef(natomwfc,nwan,nspin))
  
  coef = (0.d0,0.d0)
  wannier_energy = (0.d0,0.d0)
  wannier_occ = (0.d0,0.d0)

  wan_in(1:nwan,1:nspin) = wan_data(1:nwan,1:nspin)
  
  IF(.NOT. hwwa) THEN

     IF(use_energy_int) THEN
        do i=1,nwan
           wan_in(i,:)%bands_from = (1.d0/rytoev)*wan_in(i,:)%bands_from
           wan_in(i,:)%bands_to = (1.d0/rytoev)*wan_in(i,:)%bands_to
        end do
     END IF
     
     CALL wannier_check()
  end if

  ALLOCATE(wan_pot(nwan,nspin))
  wan_pot(1:nwan,1:nspin) = constrain_pot(1:nwan,1:nspin)
  
  !now open files to store projectors and wannier functions
  nwordwpp = nwan*nbnd*npol
  nwordwf = nwan*npwx*npol
  io_level = 1
  CALL open_buffer( iunwpp, 'wproj', nwordwpp, io_level, exst )
  CALL open_buffer( iunwf, 'wwf', nwordwf, io_level, exst )

  ! For atomic wavefunctions
  INQUIRE( UNIT = iunigk, OPENED = opnd )
  IF(.NOT. opnd) CALL seqopn( iunigk, 'igk', 'UNFORMATTED', exst )

  IF(.NOT. ALLOCATED(swfcatom)) ALLOCATE( swfcatom( npwx, natomwfc))
  
  nwordatwfc = npwx*natomwfc*npol
  INQUIRE( UNIT = iunsat, OPENED = opnd )
  IF(.NOT. opnd) CALL open_buffer( iunsat,'satwfc',nwordatwfc,io_level,exst )

  RETURN
  !
END SUBROUTINE wannier_init
