/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2003 A. Smogunov
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
subroutine gep_x(n, amt, bmt, eigen, veigen)
!
! It solves GEP: A X = lambda B X using LAPACK routines
!
  USE kinds, only : DP
  USE cond, only : delgep
  implicit none

  integer :: i, n, info, lwork
  complex(DP) :: trywork
  real(DP), allocatable :: rwork(:)
  complex(DP) ::                                   &
                 amt(n,n),  &  ! A
                 bmt(n,n),  &  ! B
                 eigen(n),  &  ! lambda
                 veigen(n,n)   ! X
  complex(DP), allocatable :: alpha(:), beta(:), work(:)

  allocate( alpha( n ) )
  allocate( beta( n ) )
  allocate( rwork( 8*n ) )
!
!  for some reason the lapack routine does not work if the diagonal elements
!  of the matrices are exactly zero. We tested these routines on
!  pc_ifc, ibmsp and origin.
!
  do i=1,n
     amt(i,i)=amt(i,i)+delgep
     bmt(i,i)=bmt(i,i)+delgep
  enddo

  call ZGGEV('N', 'V', n, amt, n, bmt, n, alpha, beta, veigen, n, veigen, &
              n, trywork, -1, rwork, info)

  lwork=abs(trywork)
  allocate( work( lwork ) )

  call ZGGEV('N', 'V', n, amt, n, bmt, n, alpha, beta, veigen, n, veigen, &
              n, work, lwork, rwork, info)

  if(info.ne.0) call errore('gep_x','error on zggev',info)
!
! lambda = alpha / beta
!
  do i=1, n
    eigen(i)=alpha(i)/beta(i)
!    write(6,'(i5, 2f40.20)') i,  DBLE(eigen(i)), AIMAG(eigen(i))
  enddo

  deallocate(work)
  deallocate(rwork)
  deallocate(beta)
  deallocate(alpha)

  return
end subroutine gep_x
