/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2003 A. Smogunov
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
subroutine hev_ab (n, amt, lda, eigen, veigen, el, eh, m)
!
! It solves EP: A x = lambda X with hermitean matrix A.
!
  USE kinds, only : DP
  implicit none
  character :: lrange
  integer ::                                  &
       n,      &  ! dimension of the matrix to be diagonalized
       m,      &  ! input:  flag
                  ! output: number of eigenvalues in the [el,eh]
       il, ih, &  ! number of lowest and highest eigenvalues
       lda,    &  ! leading dimension of A
       lwork,  &  ! aux. var.
       info       ! -1 --> all eigenvalues are computed

  integer, allocatable :: iwork(:), ifail(:)
  real(DP) ::  &
       eigen(n),    & ! eigenvalues
       el, eh,      & ! interval for eigenvalue searching
       abstol         ! accuracy for eigenvalues
  real(DP), allocatable  :: rwork(:)
  complex(DP), allocatable :: work(:)
  complex(DP) ::  &
       amt(lda, n),    &   ! A
       veigen(lda, n)      ! X

  lwork = 16*n

  allocate( work( lwork ) )
  allocate( rwork( 7*n ) )
  allocate( iwork( 5*n ) )
  allocate( ifail( n ) )

  abstol=0.d0

!
! If m=-1 find all the solutions, otherwise only el<lambda<eh
!
  if (m.eq.-1) then
     lrange='A'
  else
     lrange='V'
  endif

  call ZHEEVX('V', lrange, 'U', n, amt, lda, el, eh, il, ih, abstol,  &
               m, eigen, veigen, n, work, lwork, rwork, iwork, ifail, &
               info)
  call errore ('hev_ab','info =/= 0',abs(info))

  deallocate(rwork)
  deallocate(work)
  deallocate(iwork)
  deallocate(ifail)

  return
end subroutine hev_ab
