/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2003 A. Smogunov
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
subroutine sunitary(nchanl, nchanr, smat, info)
!
! It performs a check of unitarity of the scattering matrix
! S = {R_ij, T_ij}
! \sum R_ki^* R_kj + \sum T_ki^* R_kj = \delta_ij
!
  USE kinds, ONLY : DP
  USE io_global,  ONLY :  stdout
implicit none

  integer ::  info, i, j, k
  integer ::  nchanl, &  ! number of channels in the left tip
              nchanr     !     ------------         right tip
  real(DP), parameter :: eps=1.d-4
  real(DP) :: raux, raux1
  complex(DP) ::   &
              s, smat(nchanl+nchanr, nchanl)    ! S matrix

  info = 0
  do i = 1, nchanl
    do j = 1, nchanl
      s = 0.d0
      do k = 1, nchanl
       s = s + CONJG(smat(k,i))*smat(k,j)
      enddo
      do k = 1, nchanr
       s = s + CONJG(smat(nchanl+k,i))*smat(nchanl+k,j)
      enddo
      raux = sqrt(DBLE(s)**2 + AIMAG(s)**2)
      raux1 = raux
      if(i.eq.j) raux1 = abs(raux1-1.d0)
      if(raux1.gt.eps) then
        write(stdout, '(2i5,f12.7)') i, j, raux
        info = 1
      endif
    enddo
  enddo

  return
end subroutine sunitary

