/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

! Input/Output Tool Kit (IOTK)
! Copyright (C) 2004-2006 Giovanni Bussi
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

!------------------------------------------------------------------------------!
! Inclusion of configuration file
! Input/Output Tool Kit (IOTK)
! Copyright (C) 2004 Giovanni Bussi
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!
!------------------------------------------------------------------------------!
! CONFIGURATION FILE FOR IOTK 1.1.0 for Quantum-Espresso
!------------------------------------------------------------------------------!
! The following lines map some commonly defined system macro to the internal
! iotk macros.
! Iotk macros which are not defined take their default values.
! See the manual for a list of iotk macros.


! Generic options valid for quantum-espresso
! QE uses ranks up to four and default integer/logicals only


! some compilers do not like the following
!    #define __IOTK_REAL1 selected_real_kind(6,30)
!    #define __IOTK_REAL2 selected_real_kind(14,200)
! so we use explicit kinds
! Machine-dependent options
! Only for compilers that require some special tricks

    !
    ! proceed with a machine dependent def where available
    !



!------------------------------------------------------------------------------!



! The macros are defined with -D option or inside iotk_config.h
! The default values are set here

! Maximum rank of an array

! Minimum value used in iotk_free_unit

! Maximum value used in iotk_free_unit

! Unit for errors

! Unit for output

! Kind for header in binary files

! Maximum number of arguments to the iotk tool

! Character (or eventually string) for newline
! It may be adjusted for particular systems
! It is used only in binary files, surrounding the tags so that they can
! be easily isolated with grep
! Unix    achar(10)
! Mac-OS  achar(13)
! Windows ? (now it should be a single byte)

! Character for EOS

! These are the default kinds, which depend on the options used
! during the library compilation
! Only default characters are implemented
! For logical, integer and real types, the c precompiler
! looks for defined kinds. If no kind is found, the default
! is used as __IOTK_type1

! Complex are treated indentically to reals
! These lines map the definitions.

! Some useful check follow




subroutine iotk_stream_read_x(unit,header,setpos,getpos,ierr)
  use iotk_base
  use iotk_stream_interf
  implicit none
  integer,                                    intent(in)  :: unit
  integer(iotk_header_kind),                  intent(out) :: header
  integer,                          optional, intent(in)  :: setpos
  integer,                          optional, intent(out) :: getpos
  integer,                          optional, intent(out) :: ierr
  integer :: aa(1)
  call iotk_stream_read(unit,header,aa,setpos,getpos,.true.,ierr)
end subroutine iotk_stream_read_x



subroutine iotk_stream_read_LOGICAL1(unit,header,val,setpos,getpos,noval,ierr)
  use iotk_base
  use iotk_error_interf
  implicit none
  integer,                                      intent(in)  :: unit
  integer(iotk_header_kind),                    intent(out) :: header
  LOGICAL(kind=iotk_LOGICAL1),      intent(out) :: val(:)
  integer,                            optional, intent(in)  :: setpos
  integer,                            optional, intent(out) :: getpos
  logical,                            optional, intent(in)  :: noval
  integer,                            optional, intent(out) :: ierr
  logical :: lnoval
  integer :: ierrl
  lnoval = .false.
  if(present(noval)) lnoval = noval
  ierrl = 0
  header = 0
  getpos = 0
  call iotk_error_issue(ierrl,"iotk_stream_read","iotk_stream.f90",119)
call iotk_error_msg(ierrl,"CVS Revision: 1.8 ")
call iotk_error_msg(ierrl,'Streams are not implemented')
1 continue
  if(present(ierr)) then
    ierr = ierrl
  else
    if(ierrl/=0) call iotk_error_handler(ierrl)
  end if
end subroutine iotk_stream_read_LOGICAL1




subroutine iotk_stream_read_INTEGER1(unit,header,val,setpos,getpos,noval,ierr)
  use iotk_base
  use iotk_error_interf
  implicit none
  integer,                                      intent(in)  :: unit
  integer(iotk_header_kind),                    intent(out) :: header
  INTEGER(kind=iotk_INTEGER1),      intent(out) :: val(:)
  integer,                            optional, intent(in)  :: setpos
  integer,                            optional, intent(out) :: getpos
  logical,                            optional, intent(in)  :: noval
  integer,                            optional, intent(out) :: ierr
  logical :: lnoval
  integer :: ierrl
  lnoval = .false.
  if(present(noval)) lnoval = noval
  ierrl = 0
  header = 0
  getpos = 0
  call iotk_error_issue(ierrl,"iotk_stream_read","iotk_stream.f90",483)
call iotk_error_msg(ierrl,"CVS Revision: 1.8 ")
call iotk_error_msg(ierrl,'Streams are not implemented')
1 continue
  if(present(ierr)) then
    ierr = ierrl
  else
    if(ierrl/=0) call iotk_error_handler(ierrl)
  end if
end subroutine iotk_stream_read_INTEGER1




subroutine iotk_stream_read_REAL1(unit,header,val,setpos,getpos,noval,ierr)
  use iotk_base
  use iotk_error_interf
  implicit none
  integer,                                      intent(in)  :: unit
  integer(iotk_header_kind),                    intent(out) :: header
  REAL(kind=iotk_REAL1),      intent(out) :: val(:)
  integer,                            optional, intent(in)  :: setpos
  integer,                            optional, intent(out) :: getpos
  logical,                            optional, intent(in)  :: noval
  integer,                            optional, intent(out) :: ierr
  logical :: lnoval
  integer :: ierrl
  lnoval = .false.
  if(present(noval)) lnoval = noval
  ierrl = 0
  header = 0
  getpos = 0
  call iotk_error_issue(ierrl,"iotk_stream_read","iotk_stream.f90",847)
call iotk_error_msg(ierrl,"CVS Revision: 1.8 ")
call iotk_error_msg(ierrl,'Streams are not implemented')
1 continue
  if(present(ierr)) then
    ierr = ierrl
  else
    if(ierrl/=0) call iotk_error_handler(ierrl)
  end if
end subroutine iotk_stream_read_REAL1

subroutine iotk_stream_read_REAL2(unit,header,val,setpos,getpos,noval,ierr)
  use iotk_base
  use iotk_error_interf
  implicit none
  integer,                                      intent(in)  :: unit
  integer(iotk_header_kind),                    intent(out) :: header
  REAL(kind=iotk_REAL2),      intent(out) :: val(:)
  integer,                            optional, intent(in)  :: setpos
  integer,                            optional, intent(out) :: getpos
  logical,                            optional, intent(in)  :: noval
  integer,                            optional, intent(out) :: ierr
  logical :: lnoval
  integer :: ierrl
  lnoval = .false.
  if(present(noval)) lnoval = noval
  ierrl = 0
  header = 0
  getpos = 0
  call iotk_error_issue(ierrl,"iotk_stream_read","iotk_stream.f90",938)
call iotk_error_msg(ierrl,"CVS Revision: 1.8 ")
call iotk_error_msg(ierrl,'Streams are not implemented')
1 continue
  if(present(ierr)) then
    ierr = ierrl
  else
    if(ierrl/=0) call iotk_error_handler(ierrl)
  end if
end subroutine iotk_stream_read_REAL2



subroutine iotk_stream_read_COMPLEX1(unit,header,val,setpos,getpos,noval,ierr)
  use iotk_base
  use iotk_error_interf
  implicit none
  integer,                                      intent(in)  :: unit
  integer(iotk_header_kind),                    intent(out) :: header
  COMPLEX(kind=iotk_COMPLEX1),      intent(out) :: val(:)
  integer,                            optional, intent(in)  :: setpos
  integer,                            optional, intent(out) :: getpos
  logical,                            optional, intent(in)  :: noval
  integer,                            optional, intent(out) :: ierr
  logical :: lnoval
  integer :: ierrl
  lnoval = .false.
  if(present(noval)) lnoval = noval
  ierrl = 0
  header = 0
  getpos = 0
  call iotk_error_issue(ierrl,"iotk_stream_read","iotk_stream.f90",1211)
call iotk_error_msg(ierrl,"CVS Revision: 1.8 ")
call iotk_error_msg(ierrl,'Streams are not implemented')
1 continue
  if(present(ierr)) then
    ierr = ierrl
  else
    if(ierrl/=0) call iotk_error_handler(ierrl)
  end if
end subroutine iotk_stream_read_COMPLEX1

subroutine iotk_stream_read_COMPLEX2(unit,header,val,setpos,getpos,noval,ierr)
  use iotk_base
  use iotk_error_interf
  implicit none
  integer,                                      intent(in)  :: unit
  integer(iotk_header_kind),                    intent(out) :: header
  COMPLEX(kind=iotk_COMPLEX2),      intent(out) :: val(:)
  integer,                            optional, intent(in)  :: setpos
  integer,                            optional, intent(out) :: getpos
  logical,                            optional, intent(in)  :: noval
  integer,                            optional, intent(out) :: ierr
  logical :: lnoval
  integer :: ierrl
  lnoval = .false.
  if(present(noval)) lnoval = noval
  ierrl = 0
  header = 0
  getpos = 0
  call iotk_error_issue(ierrl,"iotk_stream_read","iotk_stream.f90",1302)
call iotk_error_msg(ierrl,"CVS Revision: 1.8 ")
call iotk_error_msg(ierrl,'Streams are not implemented')
1 continue
  if(present(ierr)) then
    ierr = ierrl
  else
    if(ierrl/=0) call iotk_error_handler(ierrl)
  end if
end subroutine iotk_stream_read_COMPLEX2



subroutine iotk_stream_read_CHARACTER1(unit,header,val,setpos,getpos,noval,ierr)
  use iotk_base
  use iotk_error_interf
  implicit none
  integer,                                      intent(in)  :: unit
  integer(iotk_header_kind),                    intent(out) :: header
  CHARACTER(kind=iotk_CHARACTER1,len=*),      intent(out) :: val(:)
  integer,                            optional, intent(in)  :: setpos
  integer,                            optional, intent(out) :: getpos
  logical,                            optional, intent(in)  :: noval
  integer,                            optional, intent(out) :: ierr
  logical :: lnoval
  integer :: ierrl
  lnoval = .false.
  if(present(noval)) lnoval = noval
  ierrl = 0
  header = 0
  getpos = 0
  call iotk_error_issue(ierrl,"iotk_stream_read","iotk_stream.f90",1575)
call iotk_error_msg(ierrl,"CVS Revision: 1.8 ")
call iotk_error_msg(ierrl,'Streams are not implemented')
1 continue
  if(present(ierr)) then
    ierr = ierrl
  else
    if(ierrl/=0) call iotk_error_handler(ierrl)
  end if
end subroutine iotk_stream_read_CHARACTER1




subroutine iotk_stream_backspace_x(unit,ierr)
  use iotk_base
  use iotk_error_interf
  implicit none
  integer,           intent(in)  :: unit
  integer, optional, intent(out) :: ierr
  integer :: ierrl
  ierrl=0
  call iotk_error_issue(ierrl,"iotk_stream_backspace","iotk_stream.f90",1899)
call iotk_error_msg(ierrl,"CVS Revision: 1.8 ")
call iotk_error_msg(ierrl,'Streams are not implemented')
1 continue
  if(present(ierr)) then
    ierr = ierrl
  else
    if(ierrl/=0) call iotk_error_handler(ierrl)
  end if
end subroutine iotk_stream_backspace_x

