/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

! Input/Output Tool Kit (IOTK)
! Copyright (C) 2004-2006 Giovanni Bussi
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

!------------------------------------------------------------------------------!
! Inclusion of configuration file
! Input/Output Tool Kit (IOTK)
! Copyright (C) 2004 Giovanni Bussi
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!
!------------------------------------------------------------------------------!
! CONFIGURATION FILE FOR IOTK 1.1.0 for Quantum-Espresso
!------------------------------------------------------------------------------!
! The following lines map some commonly defined system macro to the internal
! iotk macros.
! Iotk macros which are not defined take their default values.
! See the manual for a list of iotk macros.


! Generic options valid for quantum-espresso
! QE uses ranks up to four and default integer/logicals only


! some compilers do not like the following
!    #define __IOTK_REAL1 selected_real_kind(6,30)
!    #define __IOTK_REAL2 selected_real_kind(14,200)
! so we use explicit kinds
! Machine-dependent options
! Only for compilers that require some special tricks

    !
    ! proceed with a machine dependent def where available
    !



!------------------------------------------------------------------------------!



! The macros are defined with -D option or inside iotk_config.h
! The default values are set here

! Maximum rank of an array

! Minimum value used in iotk_free_unit

! Maximum value used in iotk_free_unit

! Unit for errors

! Unit for output

! Kind for header in binary files

! Maximum number of arguments to the iotk tool

! Character (or eventually string) for newline
! It may be adjusted for particular systems
! It is used only in binary files, surrounding the tags so that they can
! be easily isolated with grep
! Unix    achar(10)
! Mac-OS  achar(13)
! Windows ? (now it should be a single byte)

! Character for EOS

! These are the default kinds, which depend on the options used
! during the library compilation
! Only default characters are implemented
! For logical, integer and real types, the c precompiler
! looks for defined kinds. If no kind is found, the default
! is used as __IOTK_type1

! Complex are treated indentically to reals
! These lines map the definitions.

! Some useful check follow



module iotk_unit_list_module
use iotk_base
implicit none

type iotk_unit_list
  type (iotk_unit_list), pointer :: next
  type (iotk_unit),            pointer :: ptr
end type iotk_unit_list

logical,              save :: iotk_units_init = .false.
type(iotk_unit_list), save :: iotk_units

contains


subroutine iotk_unit_list_init(list)
  type (iotk_unit_list), intent(out) :: list
  nullify(list%ptr)
  nullify(list%next)
end subroutine iotk_unit_list_init

subroutine iotk_unit_list_destroy(list)
  type (iotk_unit_list), intent(inout) :: list
  type (iotk_unit_list), pointer       :: this,next
  if(.not.associated(list%next)) return
  this=>list%next
  do
    if(associated(this%ptr))deallocate(this%ptr)
    next=>this%next
    deallocate(this)
    if(.not.associated(next)) exit
    this=>next
  end do
end subroutine iotk_unit_list_destroy

subroutine iotk_unit_list_add(list,ptr)
  type (iotk_unit_list), intent(inout) :: list
  type (iotk_unit),            pointer       :: ptr
  type (iotk_unit_list), pointer       :: this
  allocate(this)
  this%next => list%next
  list%next => this
  allocate(this%ptr)
  ptr => this%ptr
end subroutine iotk_unit_list_add

subroutine iotk_unit_list_del(list,ptr)
  type (iotk_unit_list), intent(inout) :: list
  type (iotk_unit),            pointer       :: ptr
  type (iotk_unit_list), pointer       :: this,next_save
  if(.not.associated(list%next)) return
  if(associated(list%next%ptr,ptr)) then
    deallocate(list%next%ptr)
    next_save => list%next%next
    deallocate(list%next)
    list%next => next_save
    nullify(ptr)
    return
  end if
  this => list%next
  do
    if(.not.associated(this%next)) return
    if(associated(this%next%ptr,ptr)) exit
    this => this%next
  end do
  deallocate(this%next%ptr)
  next_save => this%next%next
  deallocate(this%next)
  this%next => next_save
  nullify(ptr)
end subroutine iotk_unit_list_del


 subroutine iotk_unit_list_search(list,ptr,unit)
   type (iotk_unit_list), intent(in) :: list
   type (iotk_unit),            pointer    :: ptr
 
  integer, optional,intent(in) :: unit

   type (iotk_unit_list), pointer    :: this
   nullify(ptr)
   this => list%next
   if(.not.associated(this)) return
   do
     if(.not.associated(this%ptr)) goto 1000


     if(present(unit)) then
       if(this%ptr%unit /= unit) goto 1000
     end if


     ptr => this%ptr
     exit
1000 continue
     if(.not.associated(this%next)) exit
     this => this%next
   end do
 end subroutine iotk_unit_list_search
 

end module iotk_unit_list_module
