/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

! Copyright (C) 2001-2003 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .


MODULE lr_dav_variables
  !--------------------------------------------------------------------------
  ! ... sets the dimensions of the variables required in the
  ! ... lr_davidson algorithm
  !--------------------------------------------------------------------------
  ! Created by Xiaochuan Ge (Oct.2012)
  USE kinds,                ONLY : dp

  IMPLICIT NONE
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Parameters
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  real (dp), PARAMETER :: zero=1.0E-10
  real (dp), PARAMETER :: PI = 3.14159265d0

  
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Namelist
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  INTEGER :: num_eign, num_init, num_basis_max,max_iter,&
             p_nbnd_occ,p_nbnd_virt ! number of occ and virt bands for projection
  REAL(kind=dp) :: residue_conv_thr, reference, close_pre, broadening,start,&
                   finish,step,turn2planb, vccouple_shift
  logical :: precondition,dav_debug, single_pole,&
             sort_contr,diag_of_h,print_spectrum,if_check_orth,&
             if_random_init,if_check_her,poor_of_ram,poor_of_ram2,&
             conv_assistant,if_dft_spectrum,lplot_drho

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !Global Variables
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  integer , allocatable :: &
        vc_couple(:,:),&  ! for calculating the optimal initial vector
        eign_value_order(:), energy_dif_order(:)

  LOGICAL :: dav_conv
  LOGICAL :: ploted(3)=.false., done_calc_R=.false.
  real(dp) :: max_res
  logical, allocatable :: kill_left(:),kill_right(:) 
  INTEGER :: num_basis_old, &   ! num of basis that has been applied Liouvillian
             num_basis,toadd,       &   ! total number of basis
             lwork,info,&         ! for calling lapack routine 
             dav_iter,num_basis_tot  

  COMPLEX (DP), ALLOCATABLE ::  &
        vec_b(:,:,:,:), svec_b(:,:,:,:),   & ! vectors for the basis of Liouvillian
        left_full(:,:,:,:), &
        right_full(:,:,:,:),&  ! left and right appro. eigen vector of D*C in full space
        left_res(:,:,:,:), &  
        right_res(:,:,:,:), &  ! left and right residual of the appr. eigen-v
        vecwork(:,:,:), svecwork(:,:,:), & ! Working space for supervectors
        left2(:),right2(:), & ! square of the module of left_res and right_res
        M_C(:,:),M_D(:,:),M(:,:),& ! Matrix represented in the sub space
        ground_state(:,:,:),&
        C_vec_b(:,:,:,:),       &  ! D and C applied to vec_b 
        D_vec_b(:,:,:,:),       &  ! D and C applied to vec_b. see documentation for explaination of D&C
        Fx(:,:),Fy(:,:)
        
  real(dp) :: FxR(3), FyR(3)

  REAL (DP), ALLOCATABLE :: &
        work(:) ,&            ! global work space for linear algorithm routine
        left_M(:,:),        &
        right_M(:,:),       & ! left and right appro. eigen vectors of
        eign_value(:,:),&
        M_shadow_avatar(:,:),&
        inner_matrix(:,:),&
        tr_energy(:),&
        energy_dif(:),contribute(:,:),&
        chi_dav(:,:),total_chi(:),norm_F(:),&
        omegal(:),omegar(:)
  
end module lr_dav_variables
