/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!-----------------------------------------------------------------------
SUBROUTINE lr_read_d0psi()
  !---------------------------------------------------------------------
  ! ... reads in and stores the vectors necessary to
  ! ... restart the Lanczos recursion
  !---------------------------------------------------------------------
  ! Modified by Osman Baris Malcioglu (2009)
  !
  !
  USE klist,                ONLY : nks,degauss
  USE io_files,             ONLY : prefix, diropn, tmp_dir, wfc_dir
  USE lr_variables,         ONLY : d0psi, n_ipol,LR_polarization
  USE lr_variables,         ONLY : nwordd0psi, iund0psi
  USE wvfct,                ONLY : nbnd, npwx,et
  USE lr_variables,   ONLY : lr_verbosity,restart
   USE io_global,      ONLY : stdout
 !
  IMPLICIT NONE
  !
  ! local variables
  INTEGER :: ip
  CHARACTER(len=6), EXTERNAL :: int_to_char
  LOGICAL :: exst
  CHARACTER(len=256) :: tmp_dir_saved
  !
  WRITE( stdout, '(/5x,"Reading d0psi")' )

  nwordd0psi = 2 * nbnd * npwx * nks
  !
  ! This is a parallel read, done in wfc_dir
  tmp_dir_saved = tmp_dir
  IF ( wfc_dir /= 'undefined' ) tmp_dir = wfc_dir
  DO ip=1,n_ipol
     !
     IF (n_ipol==1) THEN
       CALL diropn ( iund0psi, 'd0psi.'//trim(int_to_char(LR_polarization)), nwordd0psi, exst)
       IF (.not.exst .and. wfc_dir /= 'undefined') THEN
         WRITE( stdout, '(/5x,"Attempting to read d0psi from outdir instead of wfcdir")' )
         CLOSE( UNIT = iund0psi)
         tmp_dir = tmp_dir_saved
         CALL diropn ( iund0psi, 'd0psi.'//trim(int_to_char(LR_polarization)), nwordd0psi, exst)
         IF (.not.exst) CALL errore('lr_read_d0psi', trim( prefix )//'.d0psi.'//trim(int_to_char(LR_polarization))//' not found',1)
      ENDIF
     ENDIF
     IF (n_ipol==3) THEN
         CALL diropn ( iund0psi, 'd0psi.'//trim(int_to_char(ip)), nwordd0psi, exst)
       IF (.not.exst .and. wfc_dir /= 'undefined') THEN
         WRITE( stdout, '(/5x,"Attempting to read d0psi from outdir instead of wfcdir")' )
         CLOSE( UNIT = iund0psi)
         tmp_dir = tmp_dir_saved
         CALL diropn ( iund0psi, 'd0psi.'//trim(int_to_char(LR_polarization)), nwordd0psi, exst)
         IF (.not.exst) CALL errore('lr_read_d0psi', trim( prefix )//'.d0psi.'//trim(int_to_char(ip))//' not found',1)
       ENDIF
     ENDIF
     !
     CALL davcio(d0psi(1,1,1,ip),nwordd0psi,iund0psi,1,-1)
     !
     CLOSE( UNIT = iund0psi)
     !
  ENDDO
  ! End of file i/o
  tmp_dir = tmp_dir_saved
  !
END SUBROUTINE lr_read_d0psi
!-----------------------------------------------------------------------
