/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2004-2013 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!--------------------------------------------------------------------

SUBROUTINE stop_lr( full_run  )
  !----------------------------------------------------------------------------
  !
  ! ... Synchronize processes before stopping.
  !
  ! Modified by O. Baris Malcioglu (2009)
  USE kinds, ONLY : DP
  USE mp_global,  ONLY : mp_global_end
  !
  USE parallel_include
  USE lr_variables,         ONLY : n_ipol, LR_polarization, beta_store
  USE lr_variables,         ONLY :  gamma_store, zeta_store, norm0, code
  USE lr_variables,         ONLY : lr_verbosity, itermax, bgz_suffix
  USE io_global,            ONLY : ionode
  USE io_files,             ONLY : tmp_dir, prefix
  USE io_global,            ONLY : stdout
  USE environment,          ONLY : environment_end
  ! For gaussian cube file
  USE ions_base,  ONLY : nat, ityp, atm, ntyp => nsp, tau
  USE cell_base,  ONLY : celldm, at, bg
  !
  IMPLICIT NONE
  !
  LOGICAL, INTENT(IN) :: full_run
  !
  CHARACTER(len=6), EXTERNAL :: int_to_char
  !
  CHARACTER(len=256) :: filename
  !
  INTEGER :: ip,i,j
  !
  !

  IF (lr_verbosity > 5) THEN
    WRITE(stdout,'("<stop_lr>")')
  ENDIF
  ! I write the beta gamma and z coefficents to output directory for
  ! easier post processing. These can also be read from the output log file
  IF(full_run) THEN
  IF (ionode) THEN
  !
  DO ip=1,n_ipol
   IF (n_ipol==3) filename = trim(prefix) // trim(bgz_suffix) // trim(int_to_char(ip))
   IF (n_ipol==1) filename = trim(prefix) // trim(bgz_suffix) // trim(int_to_char(LR_polarization))
   filename = trim(tmp_dir) // trim(filename)
  !
  !
  OPEN (158, file = filename, form = 'formatted', status = 'replace')
  !
  WRITE(158,*) itermax
  !
  norm0(ip)=beta_store(ip,1)
  WRITE(158,*) norm0(ip)
  !
  DO i=1,itermax-1
     !
     WRITE(158,*) beta_store(ip,i+1)
     WRITE(158,*) gamma_store(ip,i+1)
     !This is absolutely necessary for cross platform compatibilty
     DO j=1,n_ipol
      WRITE(158,*) zeta_store (ip,j,i)
     ENDDO
     !
  ENDDO
  WRITE(158,*) beta_store(ip,itermax)  ! XC: This two faked values will not be
  WRITE(158,*) gamma_store(ip,itermax) ! really used in spectrum calculation
  DO j=1,n_ipol                                               
    WRITE(158,*) zeta_store (ip,j,itermax)                    
  ENDDO
  !
  CLOSE(158)
  !
  ENDDO
  ENDIF
  ENDIF
  !
  !   Deallocate lr variables
  !
  CALL lr_dealloc()
  !
  !
  CALL environment_end(code)
  !
  CALL mp_global_end( )
  !
  !
  STOP
  !
  !
END SUBROUTINE stop_lr
