/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2004 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!---------------------------------------------------------------
subroutine esic
  !---------------------------------------------------------------
  !
  use kinds, only: dp
  use radial_grids, only: ndmx
  use ld1inc, only: grid, etot, ehrt, ecxc, ekin, rel, dhrsic, dxcsic, &
                    nwf, ll,  oc, psi 
 
  implicit none
  ! output
  ! local
  integer:: n, i
  real(DP) :: int_0_inf_dr,deksic  
  real(DP) :: work1(ndmx),v(ndmx),vsic(ndmx)
  external int_0_inf_dr
  !
  deksic = 0.0_DP
  dhrsic = 0.0_DP
  dxcsic = 0.0_DP
  do n=1,nwf
     call sic_correction(n,v,vsic,work1)
     if (rel.eq.2) then
        do i=1,grid%mesh
           v(i)=v(i)*(psi(i,1,n)**2+psi(i,2,n)**2)
           vsic(i)=vsic(i)*(psi(i,1,n)**2+psi(i,2,n)**2)
        end do
     else
        do i=1,grid%mesh
           v(i)=v(i)*psi(i,1,n)**2
           vsic(i)=vsic(i)*psi(i,1,n)**2
        enddo
     endif
     deksic = deksic +  &
          &      oc(n)*int_0_inf_dr(vsic,grid,grid%mesh,2*(ll(n)+1))
     dhrsic = dhrsic - 0.5_DP*oc(n)*int_0_inf_dr(v    ,grid,grid%mesh,2)
     dxcsic = dxcsic -        oc(n)*int_0_inf_dr(work1,grid,grid%mesh,2)
  enddo
  !
  ekin=ekin+deksic
  ehrt=ehrt
  ecxc=ecxc+dxcsic+dhrsic
  etot=etot+dhrsic+dxcsic+deksic
  !
  return
end subroutine esic
