/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2004-2008 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!---------------------------------------------------------------
function int_0_inf_dr(f,grid,mesh,nst)
  !---------------------------------------------------------------
  !
  !      integral of f from 0 to infinity
  !      f is given on a logarithmic mesh. 
  !      f(r) is assumed to be proportional to r**nst for small r
  !
  use kinds, only : DP
  use radial_grids, only: radial_grid_type, series
  implicit none
  !
  ! I/O variables
  !
  integer, intent(in) :: mesh, nst
  real(DP), intent(in):: f(mesh)
  type(radial_grid_type), intent(in) :: grid
  real(DP) :: int_0_inf_dr
  !
  ! local variables
  !
  real(DP):: fs(4), b(4), sum1
  integer :: i
  !
  ! series development: contribution for small r
  !
  if (mesh > grid%mesh) &
     call errore('int_0_inf_dr','value of mesh is larger than expected',mesh)

  do i=1,4
     fs(i)=f(i)/grid%r(i)**nst
  end do
  call series(fs,grid%r,grid%r2,b)
  int_0_inf_dr = ( b(1)/(nst+1) + grid%r(1)* &
                  (b(2)/(nst+2) + grid%r(1)*b(3)/(nst+3)) ) * grid%r(1)**(nst+1)
  !
  ! simpson integration (logarithmic mesh: dr ==> r dx)
  !
  sum1=0.0_DP
  do i=1,mesh-2,2
     sum1 = sum1 + f(i)*grid%r(i)+4.0_dp*f(i+1)*grid%r(i+1)+f(i+2)*grid%r(i+2)
  end do
  int_0_inf_dr = int_0_inf_dr + sum1*grid%dx/3.0_DP

  return
end function int_0_inf_dr
