/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2004 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-------------------------------------------------------------------------
subroutine newd_at ( )
  !-------------------------------------------------------------------------
  !
  !     this routine computes the new D coefficients
  !
  !
  use kinds, only : dp
  use radial_grids, only : ndmx
  use ld1inc, only : ddd, bmat, nbeta, nspin, lls, jjs, ikk, qvan, vpstot, &
                     grid, pseudotype, lpaw, which_augfun, qvanl
  implicit none

  integer :: &
       ib,jb,n,is,nst

  real(DP) :: &
       int_0_inf_dr, &   ! the integral function
       gi(ndmx)          ! the gi function

  !
  !    screening the D coefficients
  !
  if (pseudotype == 3) then
     do ib=1,nbeta
        do jb=1,ib
           if (lls(ib).eq.lls(jb).and.abs(jjs(ib)-jjs(jb)).lt.1.0e-7_dp) then
              nst=(lls(ib)+1)*2
              do is=1,nspin
                 IF (which_augfun=='PSQ') then
                    do n=1,ikk(ib)
                       gi(n)=qvanl(n,ib,jb,0)*vpstot(n,is)
                    enddo
                 ELSE
                    do n=1,ikk(ib)
                       gi(n)=qvan(n,ib,jb)*vpstot(n,is)
                    enddo
                 ENDIF
                 ddd(ib,jb,is)= bmat(ib,jb) &
                      + int_0_inf_dr(gi,grid,ikk(ib),nst)
                 ddd(jb,ib,is)=ddd(ib,jb,is)
              enddo
           endif
        enddo
     enddo
  else if (pseudotype == 2) then
     !
     !    non-US separable PP case: just copy unscreened D coeffs
     !
     do is=1,nspin
        ddd(:,:,is)= bmat(:,:)
     enddo
  endif

  return
end subroutine newd_at
