/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2007 Quantum ESPRESSO group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!-----------------------------------------------------------------------
subroutine set_rc_rv()
  !-----------------------------------------------------------------------
  !
  !      input : all-electron wavefunctions + valence states
  !      output: separated core and valence charges 
  !
  use kinds, only : dp
  use ld1_parameters, only : nwfx
  
  use ld1inc, only : grid, aeccharge, aevcharge, nwf, oc, isw, rel, psi, &
                     core_state
  implicit none

  integer :: n, ns, is
  !
  !      calculates core charge density
  !
  aevcharge=0.0_DP
  aeccharge=0.0_DP
  do n=1,grid%mesh
     do ns=1,nwf
        if (oc(ns)>0.0_DP) then
           is=isw(ns)
           if (rel==2) then
              if (core_state(ns)) then
                 aeccharge(n)=aeccharge(n) &
                              +oc(ns)*( psi(n,1,ns)**2 + psi(n,2,ns)**2 )
              else
                 aevcharge(n,is)=aevcharge(n,is)+oc(ns)*(psi(n,1,ns)**2 &
                                                       + psi(n,2,ns)**2)
              endif
           else
              if (core_state(ns)) then
                 aeccharge(n) = aeccharge(n) + oc(ns)*psi(n,1,ns)**2
              else
                 aevcharge(n,is) = aevcharge(n,is) + oc(ns)*psi(n,1,ns)**2
              endif
           endif
        endif
     enddo
  enddo
  return
end subroutine set_rc_rv
