/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!-----------------------------------------------------------------------
subroutine set_sl3(sl3,lmax)
!-----------------------------------------------------------------------
   ! compute sl3(l1,l2,l3)=\int pl(x,l1) pl(x,l2) pl(x,l3) dx
   !
   use kinds, only: DP
   implicit none
   !
   ! I/O variables
   !
   integer :: lmax
   real (DP) :: sl3(0:2*lmax,0:2*lmax,0:2*lmax)
   !
   ! local variables
   !
   integer :: lmax2, l1, l2, l3, l1i, l1f

   lmax2 = 2*lmax
   do l1=0,lmax2
      do l2=0,lmax2
         do l3=0,lmax2
            sl3(l1,l2,l3) = 0.0d0
         end do
      end do
   end do

   do l3=0,lmax2
      sl3(l3,0,l3) = 2.0d0/(2.0d0*l3+1.0d0)
      sl3(0,l3,l3) = 2.0d0/(2.0d0*l3+1.0d0)
      do l2=1,lmax2
         l1i = max(abs(l2-l3),1)
         l1f = lmax2-abs(lmax2-l2-l3)
         if (l1f.eq.lmax2) then
            sl3(lmax2,l2,l3) = (2.0d0*l2-1.0d0)/(2.0d0*l1f+1.0d0) *  &
                               l1f*sl3(l1f-1,l2-1,l3)/l2
            l1f=l1f-1
         end if
         do l1=l1i,l1f
            sl3(l1,l2,l3) = (2.0d0*l2-1.0d0)/(2.0d0*l1+1.0d0) * &
               ((l1+1.0d0)*sl3(l1+1,l2-1,l3)+l1*sl3(l1-1,l2-1,l3))/l2
            if (l2.gt.1) sl3(l1,l2,l3) = sl3(l1,l2,l3) -(l2-1.0d0)/l2 * &
                                        sl3(l1,l2-2,l3)
         end do
      end do
   end do
   return
end subroutine set_sl3
