/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2004 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
!---------------------------------------------------------------
subroutine sic_correction(n,vhn1,vhn2,egc) 
  !---------------------------------------------------------------
  !   set up the orbital-dependent selfconsistent potential generated
  !   by the n-th wavefunction - for self-interaction correction
  !
  use kinds, only : dp
  use radial_grids, only : ndmx
  use constants, only: e2, fpi
  use ld1inc, only : nspin, lsd, rel, nlcc, rhoc, grid, psi
  use funct, only: dft_is_gradient
  use radial_grids, only: hartree
  implicit none
  integer :: n
  real(DP):: vhn1(ndmx),vhn2(ndmx), egc(ndmx)
  REAL(dp) :: & ! compatibility with metaGGA - not yet used
       tau(ndmx) = 0.0_dp, vtau(ndmx) = 0.0_dp
  !
  integer :: i
  real(DP):: rh(2), rhc, vxcp(2), excp
  real(DP):: vgc(ndmx,2),  egc0(ndmx), rhotot(ndmx,2)
  logical :: gga

  vhn1=0.0_dp
  vhn2=0.0_dp
  gga=dft_is_gradient()
  nspin=1
  if (lsd.eq.1) nspin=2
  !
  !   compute hartree potential with the charge of orbital n
  !
  rhotot=0.0_dp
  if (rel.eq.2) then
     do i=1,grid%mesh
        rhotot(i,1)=psi(i,1,n)**2+psi(i,2,n)**2
     enddo
  else
     do i=1,grid%mesh
        rhotot(i,1)=psi(i,1,n)**2
     enddo
  endif
  !call hartree(0,2*(ll(n)+1),grid%mesh,grid,rhotot,vhn1)
  call hartree(0,2,grid%mesh,grid,rhotot,vhn1)
  !
  !    add exchange and correlation potential: LDA or LSDA terms
  !
  rhc=0.0_dp
  rh=0.0_dp
  do i=1,grid%mesh
     vhn1(i) = e2*vhn1(i)
     rh(1) = rhotot(i,1)/grid%r2(i)/fpi
     if (nlcc) rhc = rhoc(i)/grid%r2(i)/fpi
     call vxc_t(lsd,rh,rhc,excp,vxcp)
     vhn2(i)= vhn1(i)+vxcp(1)
     egc(i)= excp*rhotot(i,1)
  end do

  if (.not.gga) return
  !
  !   add gradient-correction terms to exchange-correlation potential
  !
  egc0=egc
  call vxcgc ( ndmx, grid%mesh, nspin, grid%r, grid%r2, rhotot, rhoc, &
       vgc, egc, tau, vtau, 1)
  do i=1,grid%mesh
     vhn2(i)=vhn2(i)+vgc(i,1)
     egc(i)=egc(i)*grid%r2(i)*fpi+egc0(i)
  enddo
  return
end subroutine sic_correction
