/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2004 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!
! Copyright (C) 2001 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
! packs the potential and d coefficients into one array
! TEMPORARY: should be merged with vpack
!
subroutine vdpack (ndim, ndimx, nwf, nwfx, nspin, v, d, vd, sflag)
  use kinds, ONLY: DP
  implicit none
  integer :: ndim, ndimx, nwf, nwfx, nspin, is, n, i, ns, ns1
  character(len=4) :: sflag
  real(DP) :: v(ndimx,2), d(nwfx,nwfx,2), vd(ndimx*2+nwfx*nwfx*2)
  select case (sflag)
  case ("PACK")
     i=1
     do is=1, nspin
        do n=1, ndim
           vd(i)=v(n,is)
           i=i+1
        end do
        do ns=1, nwf
           do ns1=1, nwf
              vd(i)=d(ns,ns1,is)
              i=i+1
           end do
        end do
     end do
  case ("UNDO")
     i=1
     do is=1, nspin
        do n=1, ndim
           v(n,is)=vd(i)
           i=i+1
        end do
        do ns=1, nwf
           do ns1=1, nwf
              d(ns,ns1,is)=vd(i)
              i=i+1
           end do
        end do
     end do
  case default
     call errore ('vdpack', ' wrong flag ', 1)
  end select
  return
end subroutine vdpack
