/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
!---------------------------------------------------------------------
subroutine write_ae_pseudo
  !---------------------------------------------------------------------
  !
  ! This routine generates a UPF file with a local Coulomb potential
  ! and the all-electron starting wave-functions. It allows to make
  ! an all-electron calculation with plane-waves.
  !
  use kinds,     only : dp
  use io_global, only : ionode, ionode_id
  use mp,        only : mp_bcast
  use ld1inc, only : file_pseudopw, rel, grid, &
                     etot, zed, nwf,  el,  ll,  oc,  psi,  rho, &
                     etots,zval,nwfts,elts,llts,octs,phits,rhos, &
                     nwfs, lloc, rcloc, iswitch, nlcc, lpaw, ecutrho, &
                     ecutwfc, nbeta, lmax
  implicit none

  integer :: ios,   &  ! I/O control
             iunps     ! the unit with the pseudopotential
  character (len=2) :: atom
  character (len=2), external :: atom_name

  IF (iswitch /= 1 ) call errore('write_ae_pseudo','wrong iswitch',1)
  
  atom = atom_name(nint(zed))
  IF ( atom(1:1) == ' ' ) THEN
     file_pseudopw = atom(2:2) // '.UPF'
  ELSE
     file_pseudopw = TRIM(atom) // '.UPF'
  END IF
!  iunps=28
!  if (ionode) &
!     open(unit=iunps, file=trim(file_pseudopw), status='unknown',  &
!          form='formatted', err=50, iostat=ios)
!50  call mp_bcast(ios, ionode_id)
!  call errore('write_ae_pseudo','opening file_pseudopw',abs(ios))

  if ( rel==2 ) call errore('write_ae_pseudo','you cannot be serious!!!',rel)
  if (ionode) then
    !
    lloc = 0
    rcloc = 0.0_DP
    nwfs = 0
!    call write_pseudo_comment(iunps)  
    zval = zed
    etots= etot
    nwfts = nwf
    nbeta = 0
    nlcc = .false.
    ecutwfc=0.0_DP
    ecutrho=0.0_DP
    lpaw=.false.
    lmax=0
    elts(1:nwfts) = el(1:nwf)
    llts(1:nwfts) = ll(1:nwf)
    octs(1:nwfts) = oc(1:nwf)
    phits(1:grid%mesh,1:nwfts) = psi(1:grid%mesh,1,1:nwf )
!    call write_pseudo_header(iunps)  
!    call write_pseudo_mesh(iunps)
!    call write_pseudo_pswfc(iunps)
    rhos(1:grid%mesh,1)= rho(1:grid%mesh,1) 
!    call write_pseudo_rhoatom(iunps)  
    call ld1_writeout()
    !
!    close(iunps)
    !
  endif
  !
  return
end subroutine write_ae_pseudo
