/* Copyright (C) 1991-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */


/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */

/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */



/* wchar_t uses Unicode 10.0.0.  Version 10.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2017, fifth edition, plus
   the following additions from Amendment 1 to the fifth edition:
   - 56 emoji characters
   - 285 hentaigana
   - 3 additional Zanabazar Square characters */

!
! Copyright (C) 2004 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!--------------------------------------------------------------
subroutine write_paw_recon
  !--------------------------------------------------------------
  use kinds,     only : dp
  use io_global, only : stdout, ionode, ionode_id
  use mp,        only : mp_bcast
  use mp_world,  only : world_comm
  use ld1inc,    only : file_recon, nwf, nwfts, grid, llts, psi, phits, &
       lgipaw_reconstruction, wfc_ae_recon, wfc_ps_recon, nstoaets, &
       use_paw_as_gipaw
  implicit none
  
  integer :: i, j, n, m, l, ios, iae, isign
  real(DP) ::  wmax
  
  if (ionode) &
       open(unit=51,file=file_recon,status='unknown', err=1111, &
       iostat=ios,form='formatted')
1111 call mp_bcast(ios, ionode_id, world_comm)      
  call errore('write_result','opening 51',abs(ios))
  
  iae=nwf
  if (ionode) then
     
     write (51,*) '<PP_PAW>'
     write (51,*) nwfts
     write (51,*) '</PP_PAW>'
     do i=nwfts,1,-1
        write (51,*) '<PP_REC>'
        write (51,*) '<PP_kkbeta>'
        write (51,*) grid%mesh
        write (51,*) '</PP_kkbeta>'
        write (51,*) '<PP_L>'
        write (51,*) llts(i)
        write (51,*) '</PP_L>'
        write (51,*) '<PP_REC_AE>'
        
        IF ( lgipaw_reconstruction.and.(.not.use_paw_as_gipaw) ) THEN
           ! The data was changed in 'calculate_gipaw_orbitals()'
           psi(:grid%mesh,1,iae) = wfc_ae_recon(:grid%mesh,nstoaets(i))
        END IF
        
        ! Check sign of ae-wfct (max should be >0)
        isign=+1
        wmax=0.0_dp
        do n=1,grid%mesh
           if(abs(psi(n,1,iae)).gt.wmax.and.grid%r(n).lt.4.0_dp)then
              wmax=abs(psi(n,1,iae))
              if(psi(n,1,iae).lt.0.0_dp)then
                 isign=-1
              else
                 isign=+1
              endif
           endif
        enddo
        
        write (51,'(1p4e19.11)') (isign*psi(n,1,iae) , n=1,grid%mesh )
        write (51,*) '</PP_REC_AE>'
        write (51,*) '<PP_REC_PS>'
        
        IF ( lgipaw_reconstruction.and.(.not.use_paw_as_gipaw) ) THEN
           ! The data was changed in 'calculate_gipaw_orbitals()'
           phits(:grid%mesh,i) = wfc_ps_recon(:grid%mesh,i)
        END IF
        
        ! check sign of pseudo wfct (max should be >0)
        
        wmax=0.0_dp
        do n=1,grid%mesh
           if(abs(phits(n,i)).gt.wmax.and.grid%r(n).lt.4.0_dp)then
              wmax=abs(phits(n,i))
              if(phits(n,i).lt.0.0_dp)then
                 isign=-1
              else
                 isign=+1
              endif
           endif
        enddo
        
        write (51,'(1p4e19.11)') (isign*phits(n,i) , n=1,grid%mesh )
        write (51,*) '</PP_REC_PS>'
        write (51,*) '</PP_REC>'
        iae=iae-1
     enddo
     close(51)
  endif
  
end subroutine write_paw_recon
